package de.starface.examples.uci30.swing.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import org.apache.commons.lang.StringUtils;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.examples.uci30.swing.model.Config;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciCallEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciCallRequests;
import de.starface.integration.uci.java.v30.types.Call;
import de.starface.integration.uci.java.v30.types.CallIdChanges;
import de.starface.integration.uci.java.v30.types.Image;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.types.MessageWaitingIndicationInfo;
import de.starface.integration.uci.java.v30.values.CallState;
import de.starface.integration.uci.v30.client.UciServiceRequests;

public class CallViewModel implements UciCallEvents
{
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////
	
	private UciCallRequests callRequests;
	
	private PhoneToUseProvider phoneToUseProvider;
	private ConferenceCallViewModel conferenceCallViewModel;
	
	public CallViewModel(PhoneToUseProvider phoneToUseProvider, ConferenceCallViewModel conferenceCallViewModel)
	{
		this.phoneToUseProvider = phoneToUseProvider;
		this.conferenceCallViewModel = conferenceCallViewModel;
	}
	
	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException
	{
		this.callRequests = uciServiceRequests.getRequests(UciCallRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciCallEvents.class);
	}
	
	@Override
	public synchronized void receiveCallState(Call call)
	{
		if (call != null)
		{
			if (call.getState() == CallState.HANGUP)
			{
				removeCall(call.getId());
			}
			else
			{
				updateCall(call);
			}
		}
		conferenceCallViewModel.receiveCallState(call);
	}
	
	@Override
	@RpcAsynchronous
	public void newMessageWaitingIndicationEvent(MessageWaitingIndicationInfo mwiInfo)
	{
		if (mwiInfo != null)
		{
			updateMwiList(mwiInfo);
		}
		
	}
	
	/// View Actions
	////////////////////////////////////////////////////////////////
	
	private void updateMwiList(MessageWaitingIndicationInfo mwiInfo)
	{
		mwiInfos.clear();
		mwiInfos.add(mwiInfo.getVoiceMailBoxName() + " [id " + mwiInfo.getVoiceMailBoxId() + "]:" + " #"
			+ mwiInfo.getNewMessagesCount());
		changeSupport.firePropertyChange(PROPERTY_MWI_LIST, null, mwiInfos);
	}
	
	private void doGetImageFromDoorline()
	{
		if (!StringUtils.isEmpty(doorlineCamUrl))
		{
			BufferedImage image = null;
			try
			{
				URL url = new URL(doorlineCamUrl);
				image = javax.imageio.ImageIO.read(url);
				Icon icon = new ImageIcon(image.getScaledInstance(130, 130, java.awt.Image.SCALE_SMOOTH));
				if (icon != null)
				{
					setDoorline(icon);
				}
				
			}
			catch (IOException e)
			{
			}
		}
	}
	
	private void doOpenDoorline()
	{
		if (!StringUtils.isEmpty(doorlineDtmfCode) && !StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				callRequests.playDtmf(selectedCallId, doorlineDtmfCode);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doGetImageForHash()
	{
		if (!StringUtils.isEmpty(avatarHash) && !StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				Image image = callRequests.getImageOfHash(avatarHash);
				Icon icon = UserStateViewModel.scaleImageToIcon(image, 58);
				if (icon != null)
				{
					setAvatar(icon);
				}
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doTransferToVoicemailBox()
	{
		if (!StringUtils.isEmpty(selectedCallId) && selectedMailbox != null)
		{
			try
			{
				callRequests.transferToVoicemailBox(selectedCallId, selectedMailbox.getId());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doPlaceCallWithPhone()
	{
		if (!StringUtils.isEmpty(destinationNumber))
		{
			try
			{
				// place call sending placeCallWithPhone request
				String callId = callRequests.placeCallWithPhone(destinationNumber,
					phoneToUseProvider.getPhoneToUse(), newCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doDialVoicemailBox()
	{
		try
		{
			// Place call to voicemailBox sending placeCallWithNumber request
			callRequests.callVoicemailBox(phoneToUseProvider.getPhoneToUse());
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doPlayDtmf()
	{
		if (!StringUtils.isEmpty(dtmfDigits) && !StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				callRequests.playDtmf(selectedCallId, dtmfDigits);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doPlaceCallWithNumber()
	{
		if (!StringUtils.isEmpty(destinationNumber) && !StringUtils.isEmpty(originatingNumber))
		{
			try
			{
				// Place call sending placeCallWithNumber request
				// This request is deprecated and not documented in the UCI specification
				// due to its limitation leading to erroneous or missing communication call events
				@SuppressWarnings("deprecation")
				String callId = callRequests.placeCallWithNumber(destinationNumber, originatingNumber, newCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doPlaceConsultationCallWithPhone()
	{
		if (!StringUtils.isEmpty(destinationNumber) && !StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Place call sending placeConsultationCallWithPhone request
				String callId = callRequests.placeConsultationCallWithPhone(destinationNumber,
					phoneToUseProvider.getPhoneToUse(), newCallId, selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doTransferConsultationCall()
	{
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Transfer call sending transferConsultationCall request
				callRequests.transferConsultationCall(selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doTransferToConferenceConsultationCall()
	{
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Transfer call sending transferToConferenceConsultationCall request
				callRequests.transferToConferenceConsultationCall(selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doGetCallState()
	{
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Retrieve call state sending getCallState request
				Call call = callRequests.getCallState(selectedCallId);
				receiveCallState(call);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doGetCurrentMessageWaitingState()
	{
		try
		{
			callRequests.getCurrentMessageWaitingState();
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doHangupCall()
	{
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Hangup call sending hangupCall request
				callRequests.hangupCall(selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doHoldCall()
	{
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Park a call
				callRequests.hold(selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doGrabCallFromVoicemailBox()
	{
		try
		{
			if (!StringUtils.isEmpty(selectedCallId))
			{
				callRequests.grabVoicemailBoxCallerWithPhone(selectedCallId, phoneToUseProvider.getPhoneToUse());
			}
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doForwardCall()
	{
		
		if (!StringUtils.isEmpty(selectedCallId) && !StringUtils.isEmpty(destinationNumber))
		{
			try
			{
				// Forward call sending forwardCall request
				callRequests.forward(selectedCallId, destinationNumber);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doMonitorCall()
	{
		
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Start or stop monitoring of a call
				callRequests.monitor(selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doTransferCall()
	{
		// transfer two selected calls, using transferCall request
		// one call has to be CONNECTED, the other one PARKED
		// we have to check for this manually to pass them in the right order 
		
		if (selectedRows.size() == 2)
		{
			Call call_alpha = callTableModel.getCall(selectedRows.get(0));
			Call call_beta = callTableModel.getCall(selectedRows.get(1));
			if ((call_alpha.getState() == CallState.CONNECTED || call_beta.getState() == CallState.CONSULT)
				&& call_beta.getState() == CallState.PARKED)
			{
				try
				{
					// Transfer call sending transferCall request
					callRequests.transfer(call_alpha.getId(), call_beta.getId());
				}
				catch (UciException ex)
				{
					// Too simple error management. MUST be improved in real client!
					ex.printStackTrace();
				}
			}
			else if (call_alpha.getState() == CallState.PARKED
				&& (call_beta.getState() == CallState.CONNECTED || call_beta.getState() == CallState.CONSULT))
			{
				try
				{
					// Transfer call sending transferCall request
					callRequests.transfer(call_beta.getId(), call_alpha.getId());
				}
				catch (UciException ex)
				{
					// Too simple error management. MUST be improved in real client!
					ex.printStackTrace();
				}
			}
		}
	}
	
	private void doSwitchPhone()
	{
		
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// Switch the phone of a user
				callRequests.switchPhone(selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doConferenceCall()
	{
		ArrayList<String> callIds = new ArrayList<String>();
		for (int selectedCall : selectedRows)
		{
			String callId = callTableModel.getCall(selectedCall).getId();
			callIds.add(callId);
		}
		try
		{
			// Conference Call
			callRequests.conference(callIds, phoneToUseProvider.getPhoneToUse());
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doResumeCall()
	{
		
		if (!StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				callRequests.resume(selectedCallId, phoneToUseProvider.getPhoneToUse());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doGetCallIds()
	{
		try
		{
			// clear all calls
			callTableModel.removeAllCalls();
			conferenceCallViewModel.removeAllCalls();
			clearSelectedCallProperties();
			
			// Retrieve callIds sending getCallIds request
			List<String> callIds = callRequests.getCallIds();
			
			// get details for all calls
			for (String callId : callIds)
			{
				Call call = callRequests.getCallState(callId);
				receiveCallState(call);
			}
			
			// clear selected call if its no longer there...
			if (!StringUtils.isEmpty(selectedCallId) && !callIds.contains(selectedCallId))
			{
				clearSelectedCallProperties();
			}
			
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doParkAndOrbit()
	{
		try
		{
			if (StringUtils.isEmpty(selectedCallId))
			{
				// no call selected, try to resume a parked call on the selected P&O slot
				callRequests.parkAndOrbit("", parkAndOrbitNumber, "");
			}
			else
			{
				// if theres a parked call on this P&O slot, resume it; otherwise park the selected call
				callRequests.parkAndOrbit(selectedCallId, parkAndOrbitNumber, "");
			}
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void removeCall(String callId)
	{
		if (StringUtils.equals(selectedCallId, callId))
		{
			clearSelectedCallProperties();
		}
		callTableModel.removeCall(callId);
		conferenceCallViewModel.removeCall(callId);
	}
	
	private void updateCall(Call call)
	{
		final String callId = call.getId();
		callTableModel.updateCall(call);
		if (StringUtils.equals(selectedCallId, callId))
		{
			setSelectedCallProperties(call);
		}
		
	}
	
	private void setSelectedCallProperties(Call call)
	{
		callDetailsTableModel.updateCallDetails(call);
		setSelectedCallId(call.getId());
		setAvatarHash(call.getAvatarHash());
		doGetImageForHash();
		setDoorlineCamUrl(call.getDoorlineCamUrl());
		doGetImageFromDoorline();
		setDoorlineDtmf(call.getDoorlineDtmfCode());
	}
	
	private void clearSelectedCallProperties()
	{
		setSelectedCallId("");
		setAvatarHash("");
		setAvatar(null);
		setDoorlineCamUrl("");
		setDoorline(null);
		setDoorlineDtmf("");
		callDetailsTableModel.clearCallDetails();
	}
	
	private void doDisplaySelectedCall()
	{
		if (selectedRows.size() == 1)
		{
			Call call = callTableModel.getCall(selectedRows.get(0));
			if (call != null)
			{
				setSelectedCallProperties(call);
			}
		}
		else
		{
			clearSelectedCallProperties();
		}
	}
	
	private void doCheckSelectedCalls()
	{
		if (selectedRows.size() == 0)
		{
			setGetCallStateIsAllowed(false);
			// TODO placeCallWithPhone
			// TODO placeCallWithNumber
			setHangupCallIsAllowed(false);
			setHoldIsAllowed(false);
			setResumeIsAllowed(false);
			setForwardIsAllowed(false);
			setTransferToConferenceConsultationCallIsAllowed(false);
			setSwitchPhoneIsAllowed(false);
			setMonitorIsAllowed(false);
			setParkAndOrbitIsAllowed(true);
			setConsultCallIsAllowed(false);
			setTransferConsultCallIsAllowed(false);
			setTransferToConferenceConsultationCallIsAllowed(false);
			setGrabVoicemailBoxCallerWithPhoneIsAllowed(false);
			setTransferToVoicemailBoxIsAllowed(false);
			setPlayDtmfIsAllowed(false);
			setOpenDoorIsAllowed(false);
			setGetDoorlineImageIsAllowed(false);
			setGetAvatarIsAllowed(false);
		}
		else if (selectedRows.size() == 1)
		{
			setGetCallStateIsAllowed(true);
			// TODO placeCallWithPhone
			// TODO placeCallWithNumber
			setHangupCallIsAllowed(true);
			setHoldIsAllowed(true);
			setResumeIsAllowed(true);
			setForwardIsAllowed(true);
			setTransferIsAllowed(false);
			setSwitchPhoneIsAllowed(true);
			setMonitorIsAllowed(true);
			setParkAndOrbitIsAllowed(true);
			setConsultCallIsAllowed(true);
			setTransferConsultCallIsAllowed(true);
			setTransferToConferenceConsultationCallIsAllowed(true);
			setGrabVoicemailBoxCallerWithPhoneIsAllowed(true);
			setTransferToVoicemailBoxIsAllowed(true);
			setPlayDtmfIsAllowed(true);
			setOpenDoorIsAllowed(true);
			setGetDoorlineImageIsAllowed(true);
			setGetAvatarIsAllowed(true);
		}
		else if (selectedRows.size() == 2)
		{
			setGetCallStateIsAllowed(false);
			// TODO placeCallWithPhone
			// TODO placeCallWithNumber
			setHangupCallIsAllowed(false);
			setHoldIsAllowed(false);
			setResumeIsAllowed(false);
			setForwardIsAllowed(false);
			setTransferIsAllowed(true); // TODO further constraints
			setSwitchPhoneIsAllowed(false);
			setMonitorIsAllowed(false);
			setParkAndOrbitIsAllowed(false);
			setConsultCallIsAllowed(false);
			setTransferConsultCallIsAllowed(false);
			setTransferToConferenceConsultationCallIsAllowed(false);
			setGrabVoicemailBoxCallerWithPhoneIsAllowed(false);
			setTransferToVoicemailBoxIsAllowed(false);
			setPlayDtmfIsAllowed(false);
			setOpenDoorIsAllowed(false);
			setGetDoorlineImageIsAllowed(false);
			setGetAvatarIsAllowed(false);
		}
		else
		//default
		{
			setGetCallStateIsAllowed(false);
			// TODO placeCallWithPhone
			// TODO placeCallWithNumber
			setHangupCallIsAllowed(false);
			setHoldIsAllowed(false);
			setResumeIsAllowed(false);
			setForwardIsAllowed(false);
			setTransferIsAllowed(false);
			setSwitchPhoneIsAllowed(false);
			setMonitorIsAllowed(false);
			setParkAndOrbitIsAllowed(true);
			setGrabVoicemailBoxCallerWithPhoneIsAllowed(false);
			setTransferToVoicemailBoxIsAllowed(false);
			setPlayDtmfIsAllowed(false);
			setOpenDoorIsAllowed(false);
			setGetDoorlineImageIsAllowed(false);
			setGetAvatarIsAllowed(false);
			setTransferConsultCallIsAllowed(false);
			setTransferToConferenceConsultationCallIsAllowed(false);
		}
	}
	
	/////////////////////////////////////////////////////////////////
	// Action Listener
	/////////////////////////////////////////////////////////////////
	
	private final ListSelectionListener selectedCallsListSelectionListener = new ListSelectionListener()
	{
		public void valueChanged(ListSelectionEvent e)
		{
			doDisplaySelectedCall();
			doCheckSelectedCalls();
		}
	};
	
	private final ActionListener getMessagWaitingIndicationActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetCurrentMessageWaitingState();
		}
	};
	
	private final ActionListener transferToVoicemailBoxActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doTransferToVoicemailBox();
		}
	};
	
	private final ActionListener dialVoicemailBoxActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doDialVoicemailBox();
		}
	};
	
	private final ActionListener transferConsultationCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doTransferConsultationCall();
		}
	};
	
	private final ActionListener transferToConferenceConsultationCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doTransferToConferenceConsultationCall();
		}
	};
	
	private final ActionListener getPlaceConsultationCallWithPhoneActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doPlaceConsultationCallWithPhone();
		}
	};
	
	private final ActionListener getPlaceCallWithPhoneActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doPlaceCallWithPhone();
		}
	};
	
	private final ActionListener getPlaceCallWithNumberActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doPlaceCallWithNumber();
		}
	};
	
	private ActionListener getCallStateActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetCallState();
		}
	};
	
	private ActionListener hangupCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doHangupCall();
		}
	};
	
	private ActionListener holdCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doHoldCall();
		}
	};
	
	private ActionListener openDoorLineActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doOpenDoorline();
		}
	};
	
	private ActionListener imageFromDoorlineActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetImageFromDoorline();
		}
	};
	
	private ActionListener conferenceCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doConferenceCall();
		}
	};
	
	private ActionListener forwardCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doForwardCall();
		}
	};
	
	private ActionListener parkAndOrbitActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doParkAndOrbit();
		}
	};
	
	private ActionListener imageForHashListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetImageForHash();
		}
	};
	
	private ActionListener playDtmfActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doPlayDtmf();
		}
	};
	
	private ActionListener transferCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doTransferCall();
		}
	};
	
	private ActionListener switchPhoneButtonActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doSwitchPhone();
		}
	};
	
	private ActionListener resumeCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doResumeCall();
		}
	};
	
	private ActionListener monitorCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doMonitorCall();
		}
	};
	
	private ActionListener getCallIdsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetCallIds();
		}
	};
	
	private final ActionListener grabCallFromVoicemailBoxActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGrabCallFromVoicemailBox();
		}
	};
	
	/// View Data
	////////////////////////////////////////////////////////////////
	
	private final CallTableModel callTableModel = new CallTableModel();
	
	private final CallDetailsTableModel callDetailsTableModel = new CallDetailsTableModel();
	
	private List<Integer> selectedRows = new ArrayList<Integer>();
	
	private List<String> mwiInfos = new ArrayList<String>();
	
	private String selectedMwiInfo = "";
	
	private String destinationNumber = Config.getValue("destinationNumber");
	
	private String originatingNumber = Config.getValue("originatingNumber");
	
	private String selectedCallId = "";
	
	private String newCallId = Config.getValue("callId");
	
	private String parkAndOrbitNumber;
	
	private String dtmfDigits;
	
	private String avatarHash;
	
	private Icon avatar;
	
	private String doorlineCamUrl;
	
	private String doorlineDtmfCode;
	
	private Icon doorline;
	
	private boolean getCallStateIsAllowed = false;
	private boolean placeCallWithPhoneIsAllowed = false;
	private boolean placeCallWithNumberIsAllowed = false;
	private boolean hangupCallIsAllowed = false;
	private boolean holdIsAllowed = false;
	private boolean resumeIsAllowed = false;
	private boolean forwardIsAllowed = false;
	private boolean transferIsAllowed = false;
	private boolean transferToConferenceConsultationCallIsAllowed = false;
	private boolean switchPhoneIsAllowed = false;
	private boolean monitorIsAllowed = false;
	private boolean parkAndOrbitIsAllowed = false;
	private boolean callVoicemailBoxIsAllowed = false;
	private boolean consultCallIsAllowed = false;
	private boolean transferConsultCallIsAllowed = false;
	private boolean grabVoicemailBoxCallerWithPhoneIsAllowed = false;
	private boolean transferToVoicemailBoxIsAllowed = false;
	private boolean playDtmfIsAllowed = false;
	private boolean openDoorIsAllowed = false;
	private boolean getAvatarIsAllowed = false;
	private boolean getDoorlineImageIsAllowed = false;
	
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}
	
	private List<Mailbox> mailboxValues;
	{
		setMailboxValues(Collections.<Mailbox> emptyList());
	}
	
	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////
	
	public TableModel getCallTableModel()
	{
		return callTableModel;
	}
	
	public TableModel getCallDetailsTableModel()
	{
		return callDetailsTableModel;
	}
	
	public List<Integer> getSelectedRows()
	{
		return selectedRows;
	}
	
	public static final String PROPERTY_MWI_LIST = "mwiInfos";
	
	public List<String> getMwiInfos()
	{
		return mwiInfos;
	}
	
	public void setMwiInfos(List<String> mwiInfos)
	{
		List<String> old = this.mwiInfos;
		this.mwiInfos = mwiInfos;
		changeSupport.firePropertyChange(PROPERTY_MWI_LIST, old, mwiInfos);
	}
	
	public static final String PROPERTY_GET_CALL_STATE_IS_ALLOWED = "getCallStateIsAllowed";
	
	public boolean getGetCallStateIsAllowed()
	{
		return getCallStateIsAllowed;
	}
	
	public void setGetCallStateIsAllowed(boolean getCallStateIsAllowed)
	{
		boolean old = this.getCallStateIsAllowed;
		this.getCallStateIsAllowed = getCallStateIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_GET_CALL_STATE_IS_ALLOWED, old, getCallStateIsAllowed);
	}
	
	public static final String PROPERTY_PLACE_CALL_WITH_PHONE_IS_ALLOWED = "placeCallWithPhoneIsAllowed";
	
	public boolean getPlaceCallWithPhoneIsAllowed()
	{
		return placeCallWithPhoneIsAllowed;
	}
	
	public void setPlaceCallWithPhoneIsAllowed(boolean placeCallWithPhoneIsAllowed)
	{
		boolean old = this.placeCallWithPhoneIsAllowed;
		this.placeCallWithPhoneIsAllowed = placeCallWithPhoneIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_PLACE_CALL_WITH_PHONE_IS_ALLOWED, old,
			placeCallWithPhoneIsAllowed);
	}
	
	public static final String PROPERTY_PLACE_CALL_WITH_NUMBER_IS_ALLOWED = "placeCallWithNumberIsAllowed";
	
	public boolean getPlaceCallWithNumberIsAllowed()
	{
		return placeCallWithNumberIsAllowed;
	}
	
	public void setPlaceCallWithNumberIsAllowed(boolean placeCallWithNumberIsAllowed)
	{
		boolean old = this.placeCallWithNumberIsAllowed;
		this.placeCallWithNumberIsAllowed = placeCallWithNumberIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_PLACE_CALL_WITH_NUMBER_IS_ALLOWED, old,
			placeCallWithNumberIsAllowed);
	}
	
	public static final String PROPERTY_HANGUP_CALL_IS_ALLOWED = "hangupCallIsAllowed";
	
	public boolean getHangupCallIsAllowed()
	{
		return hangupCallIsAllowed;
	}
	
	public void setHangupCallIsAllowed(boolean hangupCallIsAllowed)
	{
		boolean old = this.hangupCallIsAllowed;
		this.hangupCallIsAllowed = hangupCallIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_HANGUP_CALL_IS_ALLOWED, old, hangupCallIsAllowed);
	}
	
	public static final String PROPERTY_HOLD_IS_ALLOWED = "holdIsAllowed";
	
	public boolean getHoldIsAllowed()
	{
		return holdIsAllowed;
	}
	
	public void setHoldIsAllowed(boolean holdIsAllowed)
	{
		boolean old = this.holdIsAllowed;
		this.holdIsAllowed = holdIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_HOLD_IS_ALLOWED, old, holdIsAllowed);
	}
	
	public static final String PROPERTY_RESUME_IS_ALLOWED = "resumeIsAllowed";
	
	public boolean getResumeIsAllowed()
	{
		return resumeIsAllowed;
	}
	
	public void setResumeIsAllowed(boolean resumeIsAllowed)
	{
		boolean old = this.resumeIsAllowed;
		this.resumeIsAllowed = resumeIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_RESUME_IS_ALLOWED, old, resumeIsAllowed);
	}
	
	public static final String PROPERTY_FORWARD_IS_ALLOWED = "forwardIsAllowed";
	
	public boolean getForwardIsAllowed()
	{
		return forwardIsAllowed;
	}
	
	public void setForwardIsAllowed(boolean forwardIsAllowed)
	{
		boolean old = this.forwardIsAllowed;
		this.forwardIsAllowed = forwardIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_FORWARD_IS_ALLOWED, old, forwardIsAllowed);
	}
	
	public static final String PROPERTY_TRANSFER_IS_ALLOWED = "transferIsAllowed";
	
	public boolean getTransferIsAllowed()
	{
		return transferIsAllowed;
	}
	
	public void setTransferIsAllowed(boolean transferIsAllowed)
	{
		boolean old = this.transferIsAllowed;
		this.transferIsAllowed = transferIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_TRANSFER_IS_ALLOWED, old, transferIsAllowed);
	}
	
	public static final String PROPERTY_TRANSFER_TO_CONFERENCE_CONSULTATION_CALL_IS_ALLOWED = "transferToConferenceConsultationCallIsAllowed";
	
	public boolean getTransferConsultCallToConferenceIsAllowed()
	{
		return transferToConferenceConsultationCallIsAllowed;
	}
	
	public void setTransferToConferenceConsultationCallIsAllowed(boolean transferToConferenceConsultationCallIsAllowed)
	{
		boolean old = this.transferToConferenceConsultationCallIsAllowed;
		this.transferToConferenceConsultationCallIsAllowed = transferToConferenceConsultationCallIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_TRANSFER_TO_CONFERENCE_CONSULTATION_CALL_IS_ALLOWED, old,
			transferToConferenceConsultationCallIsAllowed);
	}
	
	public static final String PROPERTY_SWITCH_PHONE_IS_ALLOWED = "switchPhoneIsAllowed";
	
	public boolean getSwitchPhoneIsAllowed()
	{
		return switchPhoneIsAllowed;
	}
	
	public void setSwitchPhoneIsAllowed(boolean switchPhoneIsAllowed)
	{
		boolean old = this.switchPhoneIsAllowed;
		this.switchPhoneIsAllowed = switchPhoneIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_SWITCH_PHONE_IS_ALLOWED, old, switchPhoneIsAllowed);
	}
	
	public static final String PROPERTY_MONITOR_IS_ALLOWED = "monitorIsAllowed";
	
	public boolean getMonitorIsAllowed()
	{
		return monitorIsAllowed;
	}
	
	public void setMonitorIsAllowed(boolean monitorIsAllowed)
	{
		boolean old = this.monitorIsAllowed;
		this.monitorIsAllowed = monitorIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_MONITOR_IS_ALLOWED, old, monitorIsAllowed);
	}
	
	public static final String PROPERTY_PARK_AND_ORBIT_IS_ALLOWED = "parkAndOrbitIsAllowed";
	
	public boolean getParkAndOrbitIsAllowed()
	{
		return parkAndOrbitIsAllowed;
	}
	
	public void setParkAndOrbitIsAllowed(boolean parkAndOrbitIsAllowed)
	{
		boolean old = this.parkAndOrbitIsAllowed;
		this.parkAndOrbitIsAllowed = parkAndOrbitIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_PARK_AND_ORBIT_IS_ALLOWED, old, parkAndOrbitIsAllowed);
	}
	
	public static final String CALL_VOICEMAIL_BOX_IS_ALLOWED = "callVoicemailBoxIsAllowed";
	
	public boolean getCallVoicemailBoxIsAllowed()
	{
		return callVoicemailBoxIsAllowed;
	}
	
	public void setCallVoicemailBoxIsAllowed(boolean callVoicemailBoxIsAllowed)
	{
		boolean old = this.callVoicemailBoxIsAllowed;
		this.callVoicemailBoxIsAllowed = callVoicemailBoxIsAllowed;
		changeSupport.firePropertyChange(CALL_VOICEMAIL_BOX_IS_ALLOWED, old, callVoicemailBoxIsAllowed);
	}
	
	public static final String CONSULT_CALL_IS_ALLOWED = "consultCallIsAllowed";
	
	public boolean getConsultCallIsAllowed()
	{
		return consultCallIsAllowed;
	}
	
	public void setConsultCallIsAllowed(boolean consultCallIsAllowed)
	{
		boolean old = this.consultCallIsAllowed;
		this.consultCallIsAllowed = consultCallIsAllowed;
		changeSupport.firePropertyChange(CONSULT_CALL_IS_ALLOWED, old, consultCallIsAllowed);
	}
	
	public static final String TRANSFER_CONSULT_CALL_IS_ALLOWED = "transferConsultCallIsAllowed";
	
	public boolean getTransferConsultCallIsAllowed()
	{
		return transferConsultCallIsAllowed;
	}
	
	public void setTransferConsultCallIsAllowed(boolean transferConsultCallIsAllowed)
	{
		boolean old = this.transferConsultCallIsAllowed;
		this.transferConsultCallIsAllowed = transferConsultCallIsAllowed;
		changeSupport.firePropertyChange(TRANSFER_CONSULT_CALL_IS_ALLOWED, old, transferConsultCallIsAllowed);
	}
	
	public static final String GRAB_VOICEMAIL_BOX_CALLER_WITH_PHONE_IS_ALLOWED = "grabVoicemailBoxCallerWithPhoneIsAllowed";
	
	public boolean getGrabVoicemailBoxCallerWithPhoneIsAllowed()
	{
		return grabVoicemailBoxCallerWithPhoneIsAllowed;
	}
	
	public void setGrabVoicemailBoxCallerWithPhoneIsAllowed(boolean grabVoicemailBoxCallerWithPhoneIsAllowed)
	{
		boolean old = this.grabVoicemailBoxCallerWithPhoneIsAllowed;
		this.grabVoicemailBoxCallerWithPhoneIsAllowed = grabVoicemailBoxCallerWithPhoneIsAllowed;
		changeSupport.firePropertyChange(GRAB_VOICEMAIL_BOX_CALLER_WITH_PHONE_IS_ALLOWED, old,
			grabVoicemailBoxCallerWithPhoneIsAllowed);
	}
	
	public static final String PROPERTY_TRANSFER_TO_VOICEMAIL_BOX_IS_ALLOWED = "transferToVoicemailBoxIsAllowed";
	
	public boolean getTransferToVoicemailBoxIsAllowed()
	{
		return transferToVoicemailBoxIsAllowed;
	}
	
	public void setTransferToVoicemailBoxIsAllowed(boolean transferToVoicemailBoxIsAllowed)
	{
		boolean old = this.transferToVoicemailBoxIsAllowed;
		this.transferToVoicemailBoxIsAllowed = transferToVoicemailBoxIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_TRANSFER_TO_VOICEMAIL_BOX_IS_ALLOWED, old,
			transferToVoicemailBoxIsAllowed);
	}
	
	public static final String PROPERTY_PLAY_DTMF_IS_ALLOWED = "playDtmfIsAllowed";
	
	public boolean getPlayDtmfIsAllowed()
	{
		return playDtmfIsAllowed;
	}
	
	public void setPlayDtmfIsAllowed(boolean playDtmfIsAllowed)
	{
		boolean old = this.playDtmfIsAllowed;
		this.playDtmfIsAllowed = playDtmfIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_PLAY_DTMF_IS_ALLOWED, old, playDtmfIsAllowed);
	}
	
	public static final String PROPERTY_GET_DOORLINE_IMAGE_IS_ALLOWED = "getDoorlineImageIsAllowed";
	
	public boolean getGetDoorlineImageIsAllowed()
	{
		return getDoorlineImageIsAllowed;
	}
	
	public void setGetDoorlineImageIsAllowed(boolean getDoorlineImageIsAllowed)
	{
		boolean old = this.getDoorlineImageIsAllowed;
		this.getDoorlineImageIsAllowed = getDoorlineImageIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_GET_DOORLINE_IMAGE_IS_ALLOWED, old, getDoorlineImageIsAllowed);
	}
	
	public static final String PROPERTY_GET_AVATAR_IS_ALLOWED = "getAvatarIsAllowed";
	
	public boolean getGetAvatarIsAllowed()
	{
		return getAvatarIsAllowed;
	}
	
	public void setGetAvatarIsAllowed(boolean getAvatarIsAllowed)
	{
		boolean old = this.getAvatarIsAllowed;
		this.getAvatarIsAllowed = getAvatarIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_GET_AVATAR_IS_ALLOWED, old, getAvatarIsAllowed);
	}
	
	public static final String PROPERTY_OPEN_DOOR_IS_ALLOWED = "openDoorIsAllowed";
	
	public boolean getOpendoorIsAllowed()
	{
		return openDoorIsAllowed;
	}
	
	public void setOpenDoorIsAllowed(boolean openDoorIsAllowed)
	{
		boolean old = this.openDoorIsAllowed;
		this.openDoorIsAllowed = openDoorIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_OPEN_DOOR_IS_ALLOWED, old, openDoorIsAllowed);
	}
	
	public static final String PROPERTY_SELECTED_MWI = "selectedMwiInfo";
	
	public String getSelectedMwiInfo()
	{
		return selectedMwiInfo;
	}
	
	public void setSelectedMwiInfo(String selectedMwiInfo)
	{
		String old = this.selectedMwiInfo;
		this.selectedMwiInfo = selectedMwiInfo;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_MWI, old, selectedMwiInfo);
	}
	
	public static final String PROPERTY_DOORLINE = "doorline";
	
	public Icon getDoorline()
	{
		return doorline;
	}
	
	public void setDoorline(Icon doorline)
	{
		Icon old = this.doorline;
		this.doorline = doorline;
		changeSupport.firePropertyChange(PROPERTY_DOORLINE, old, doorline);
	}
	
	public static final String PROPERTY_DOORLINE_CAM_URL = "doorlineCamUrl";
	
	public String getDoorlineCamUrl()
	{
		return doorlineCamUrl;
	}
	
	public void setDoorlineCamUrl(String doorlineCamUrl)
	{
		String old = this.doorlineCamUrl;
		this.doorlineCamUrl = doorlineCamUrl;
		changeSupport.firePropertyChange(PROPERTY_DOORLINE_CAM_URL, old, doorlineCamUrl);
	}
	
	public static final String PROPERTY_DOORLINE_DTMF_CODE = "doorlineDtmfCode";
	
	public String getDoorlineDtmfCode()
	{
		return doorlineDtmfCode;
	}
	
	public void setDoorlineDtmf(String doorlineDtmfCode)
	{
		String old = this.doorlineDtmfCode;
		this.doorlineDtmfCode = doorlineDtmfCode;
		changeSupport.firePropertyChange(PROPERTY_DOORLINE_DTMF_CODE, old, doorlineDtmfCode);
	}
	
	public static final String PROPERTY_AVATAR = "avatar";
	
	public Icon getAvatar()
	{
		return avatar;
	}
	
	public void setAvatar(Icon avatar)
	{
		Icon old = this.avatar;
		this.avatar = avatar;
		changeSupport.firePropertyChange(PROPERTY_AVATAR, old, avatar);
	}
	
	public static final String PROPERTY_AVATAR_HASH = "avatarHash";
	
	public String getAvatarHash()
	{
		return avatarHash;
	}
	
	public void setAvatarHash(String avatarHash)
	{
		String old = this.avatarHash;
		this.avatarHash = avatarHash;
		changeSupport.firePropertyChange(PROPERTY_AVATAR_HASH, old, avatarHash);
	}
	
	public static final String PROPERTY_DTMF_DIGITS = "dtmfDigits";
	
	public String getDtmfDigits()
	{
		return dtmfDigits;
	}
	
	public void setDtmfDigits(String dtmfDigits)
	{
		String old = this.dtmfDigits;
		this.dtmfDigits = dtmfDigits;
		changeSupport.firePropertyChange(PROPERTY_DTMF_DIGITS, old, dtmfDigits);
	}
	
	public static final String PROPERTY_PARK_AND_ORBIT_NUMBER = "parkAndOrbitNumber";
	
	public String getParkAndOrbitNumber()
	{
		return parkAndOrbitNumber;
	}
	
	public void setParkAndOrbitNumber(String parkAndOrbitNumber)
	{
		String old = this.parkAndOrbitNumber;
		this.parkAndOrbitNumber = parkAndOrbitNumber;
		changeSupport.firePropertyChange(PROPERTY_PARK_AND_ORBIT_NUMBER, old, parkAndOrbitNumber);
	}
	
	public static final String PROPERTY_DESTINATION_NUMBER = "destinationNumber";
	
	public String getDestinationNumber()
	{
		return destinationNumber;
	}
	
	public void setDestinationNumber(String destinationNumber)
	{
		String old = this.destinationNumber;
		this.destinationNumber = destinationNumber;
		changeSupport.firePropertyChange(PROPERTY_DESTINATION_NUMBER, old, destinationNumber);
	}
	
	public static final String PROPERTY_ORIGINATING_NUMBER = "originatingNumber";
	
	public String getOriginatingNumber()
	{
		return originatingNumber;
	}
	
	public void setOriginatingNumber(String originatingNumber)
	{
		String old = this.originatingNumber;
		this.originatingNumber = originatingNumber;
		changeSupport.firePropertyChange(PROPERTY_ORIGINATING_NUMBER, old, originatingNumber);
	}
	
	public static final String PROPERTY_NEW_CALL_ID = "newCallId";
	
	public String getNewCallId()
	{
		return newCallId;
	}
	
	public void setNewCallId(String callId)
	{
		String old = this.newCallId;
		this.newCallId = callId;
		changeSupport.firePropertyChange(PROPERTY_NEW_CALL_ID, old, callId);
	}
	
	public static final String PROPERTY_SELECTED_CALL_ID = "selectedCallId";
	
	public String getSelectedCallId()
	{
		return selectedCallId;
	}
	
	public void setSelectedCallId(String selectedCallId)
	{
		String old = this.selectedCallId;
		this.selectedCallId = selectedCallId;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_CALL_ID, old, selectedCallId);
	}
	
	////////////////////////////////////////////////////////////////
	// Getter ActionListener
	////////////////////////////////////////////////////////////////
	
	public ActionListener getTransferToVoicemailBoxActionListener()
	{
		return transferToVoicemailBoxActionListener;
	}
	
	public ActionListener getTransferConsultationCallActionListenerActionListener()
	{
		return transferConsultationCallActionListener;
	}
	
	public ActionListener getTransferToConferenceConsultationCallActionListenerActionListener()
	{
		return transferToConferenceConsultationCallActionListener;
	}
	
	public ActionListener getPlaceConsultationCallWithPhoneActionListener()
	{
		return getPlaceConsultationCallWithPhoneActionListener;
	}
	
	public ActionListener getPlaceCallWithPhoneActionListener()
	{
		return getPlaceCallWithPhoneActionListener;
	}
	
	public ActionListener getDialVoicemailBoxActionListener()
	{
		return dialVoicemailBoxActionListener;
	}
	
	public ActionListener getPlaceCallWithNumberActionListener()
	{
		return getPlaceCallWithNumberActionListener;
	}
	
	public ActionListener getGetCallStateActionListener()
	{
		return getCallStateActionListener;
	}
	
	public ActionListener getGrabCallFromVoicemailBoxActionListener()
	{
		return grabCallFromVoicemailBoxActionListener;
	}
	
	public ActionListener getHangupCallActionListener()
	{
		return hangupCallActionListener;
	}
	
	public ActionListener getHoldCallActionListener()
	{
		return holdCallActionListener;
	}
	
	public ActionListener getOpenDoorLineActionListener()
	{
		return openDoorLineActionListener;
	}
	
	public ActionListener getImageForHashActionListener()
	{
		return imageForHashListener;
	}
	
	public ActionListener getImageFromDoorlineActionListener()
	{
		return imageFromDoorlineActionListener;
	}
	
	public ActionListener getConferenceCallActionListener()
	{
		return conferenceCallActionListener;
	}
	
	public ActionListener getForwardCallActionListener()
	{
		return forwardCallActionListener;
	}
	
	public ActionListener getResumeCallActionListener()
	{
		return resumeCallActionListener;
	}
	
	public ActionListener getTransferCallActionListener()
	{
		return transferCallActionListener;
	}
	
	public ActionListener getSwitchPhoneActionListener()
	{
		return switchPhoneButtonActionListener;
	}
	
	public ActionListener getGetMessagWaitingIndicationActionListener()
	{
		return getMessagWaitingIndicationActionListener;
	}
	
	public ActionListener getGetCallIdsActionListener()
	{
		return getCallIdsActionListener;
	}
	
	public ActionListener getMonitorCallActionListener()
	{
		return monitorCallActionListener;
	}
	
	public ActionListener getParkAndOrbitActionListener()
	{
		return parkAndOrbitActionListener;
	}
	
	public ActionListener getPlayDtmfActionListener()
	{
		return playDtmfActionListener;
	}
	
	public ListSelectionListener getSelectedCallsListener()
	{
		return selectedCallsListSelectionListener;
	}
	
	public void setMailboxValues(List<Mailbox> mailboxValues)
	{
		List<Mailbox> old = this.mailboxValues;
		this.mailboxValues = mailboxValues;
		if (changeSupport != null) changeSupport.firePropertyChange(PROPERTY_MAILBOX_VALUES, old, mailboxValues);
		updateMailboxRestrictionValues();
		updateSelectedMailbox();
	}
	
	public List<Mailbox> getMailboxValues()
	{
		return mailboxValues;
	}
	
	public static final String PROPERTY_MAILBOX_VALUES = "mailboxValues";
	
	public List<Mailbox> getMailboxRestrictionValues()
	{
		return mailboxRestrictionValues;
	}
	
	public static final String PROPERTY_MAILBOX_RESTRICTION_VALUES = "mailboxRestrictionValues";
	
	private List<Mailbox> mailboxRestrictionValues;
	
	public void updateMailboxRestrictionValues()
	{
		List<Mailbox> old = mailboxRestrictionValues;
		mailboxRestrictionValues = new LinkedList<Mailbox>();
		mailboxRestrictionValues.add(null);
		if (mailboxValues != null) mailboxRestrictionValues.addAll(mailboxValues);
		if (changeSupport != null) changeSupport.firePropertyChange(PROPERTY_MAILBOX_RESTRICTION_VALUES, old,
			mailboxRestrictionValues);
	}
	
	private void updateSelectedMailbox()
	{
		if (mailboxValues != null && !mailboxValues.contains(selectedMailbox))
		{
			if (mailboxValues.isEmpty())
			{
				setSelectedMailbox(null);
			}
			else
			{
				setSelectedMailbox(mailboxValues.get(0));
			}
		}
	}
	
	public Mailbox getSelectedMailbox()
	{
		return selectedMailbox;
	}
	
	public static final String PROPERTY_SELECTED_MAILBOX = "selectedMailbox";
	
	public void setSelectedMailbox(Mailbox selectedMailbox)
	{
		Mailbox old = this.selectedMailbox;
		this.selectedMailbox = selectedMailbox;
		if (changeSupport != null) changeSupport.firePropertyChange(PROPERTY_SELECTED_MAILBOX, old,
			selectedMailbox);
	}
	
	private Mailbox selectedMailbox = null;
	
	@Override
	public void callChangesIdEvent(CallIdChanges callIdChangesEvent)
	{
		String oldUuid = callIdChangesEvent.getOldId();
		String newUuid = callIdChangesEvent.getNewId();
		
		callTableModel.changeCallId(oldUuid, newUuid);
	}
}
