package de.starface.examples.uci30.swing.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.starface.com.rpc.client.RpcRequestCaller;
import de.starface.com.rpc.services.filetransfer.FileTransfer;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.com.rpc.services.filetransfer.FileTransferFailedEvent;
import de.starface.examples.uci30.swing.model.Config;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciFaxEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciFaxRequests;
import de.starface.integration.uci.java.v30.types.Fax;
import de.starface.integration.uci.java.v30.values.FaxState;
import de.starface.integration.uci.v30.client.UciServiceRequests;

public class FaxViewModel implements UciFaxEvents
{
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////
	
	private UciFaxRequests faxRequests;
	
	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException
	{
		this.faxRequests = uciServiceRequests.getRequests(UciFaxRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciFaxEvents.class);
		registerFileTransferListener();
	}
	
	@Override
	public void receiveFaxState(Fax fax)
	{
		final String faxId = fax.getId();
		if (faxId != null)
		{
			final FaxState newState = fax.getState();
			updateFax(fax);
			switch (newState)
			{
				case BUSY:
				case FAILED:
				case NOANSWER:
				case NOFAX:
				case NOFILE:
				case SENT:
					removeFax(fax);
					break;
				default:
			}
		}
	}
	
	private void registerFileTransferListener()
	{
		RpcRequestCaller<?> requestCaller = RpcRequestCaller.getRequestCallerFromProxy(faxRequests);
		FileTransferEventListener listener = new FileTransferEventListener()
		{
			
			public void fileTransferInitialized(FileTransfer fileTransfer)
			{
				setFileTransferProgress(0);
				setFileTransferStatus(fileTransfer.getRpcFile().getFileInfo().getMimeType() + " in progress");
			}
			
			public void fileTransferProgress(FileTransfer fileTransfer)
			{
				final int newProgress = new Double(fileTransfer.getProgress() * 100).intValue();
				setFileTransferProgress(newProgress);
				if (isFileTransferAbortRequested())
				{
					fileTransfer.cancel();
				}
			}
			
			public void fileTransferFailed(FileTransferFailedEvent event)
			{
				setFileTransferStatus(event.getFileTransfer().getRpcFile().getFileInfo().getMimeType()
					+ " transfer error");
			}
			
			public void fileTransferCompleted(FileTransfer fileTransfer)
			{
				setFileTransferProgress(0);
				setFileTransferStatus(fileTransfer.getRpcFile().getFileInfo().getMimeType() + " transfer finished");
			}
		};
		requestCaller.subscribeAutomaticFileUploadEvents(listener);
	}
	
	/// View Actions
	////////////////////////////////////////////////////////////////
	
	private void doGetFaxState()
	{
		if (!StringUtils.isEmpty(selectedFaxId))
		{
			try
			{
				// Retrieve call state sending getCallState request
				Fax fax = faxRequests.getFaxState(selectedFaxId);
				
				if (fax == null)
				{
					removeFax(selectedFaxId);
				}
				else
				{
					updateFax(fax);
				}
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
		else
		{
			clearFaxProperties();
		}
	}
	
	private void doGetFaxIds()
	{
		try
		{
			// Retrieve faxIds sending getFaxIds request
			List<String> faxIds = faxRequests.getFaxIds();
			
			if (!StringUtils.isEmpty(selectedFaxId) && !faxIds.contains(selectedFaxId))
			{
				removeFax(selectedFaxId);
			}
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doSendFax()
	{
		File pdfFile = new File(pdfFileName);
		File textFile = new File(textFileName);
		if (!textFile.exists() || !textFile.isFile())
		{
			textFile = null;
		}
		if (!StringUtils.isEmpty(faxNumber) && pdfFile.exists() && pdfFile.isFile())
		{
			try
			{
				setFileTransferAbortRequested(false);
				// Send a fax using the UciProxy sendFax convenience method
				faxRequests.sendFaxAsync(faxNumber, newFaxId, textFile, pdfFile);
				
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doAbortFiletransfer()
	{
		setFileTransferAbortRequested(true);
	}
	
	private void removeFax(String faxId)
	{
		if (StringUtils.equals(selectedFaxId, faxId))
		{
			setSelectedFaxId("");
			clearFaxProperties();
		}
		faxIds.remove(faxId);
		changeSupport.firePropertyChange(PROPERTY_FAX_IDS, null, faxIds);
	}
	
	private void removeFax(Fax fax)
	{
		final String faxId = fax.getId();
		if (StringUtils.equals(selectedFaxId, faxId))
		{
			setSelectedFaxId("");
			if (fax.getState() != FaxState.SENT)
			{
				clearFaxProperties();
			}
		}
		faxIds.remove(faxId);
		changeSupport.firePropertyChange(PROPERTY_FAX_IDS, null, faxIds);
	}
	
	/*
	 * file transfer is *not* associated with a fax; the displayed properties belong to the most
	 * recently initiated transfer
	 */
	private void clearFileProperties()
	{
		setFileTransferProgress(0);
		setFileTransferStatus("");
	}
	
	private void updateFax(Fax fax)
	{
		final String faxId = fax.getId();
		if (!faxIds.contains(faxId))
		{
			faxIds.add(faxId);
			changeSupport.firePropertyChange(PROPERTY_FAX_IDS, null, faxIds);
		}
		setSelectedFaxId(faxId);
		displayFaxProperties(fax);
	}
	
	private void clearFaxProperties()
	{
		setFaxState(null);
		setCalledNumber("");
		setFaxTransferProgress(0);
		setFaxTransferStatus("");
	}
	
	private void displayFaxProperties(Fax fax)
	{
		setFaxState(fax.getState());
		setCalledNumber(fax.getCalledNumber());
		displayFaxTransferProgress(fax);
		displayFaxTransferStatus(fax);
	}
	
	private void displayFaxTransferProgress(Fax fax)
	{
		if (fax.getTotalPages() == 0)
		{
			setFaxTransferProgress(0);
		}
		else
		{
			setFaxTransferProgress((100 * fax.getSentPages()) / fax.getTotalPages());
		}
	}
	
	private void displayFaxTransferStatus(Fax fax)
	{
		setFaxTransferStatus(fax.getSentPages() + " / " + fax.getTotalPages());
	}
	
	private final ActionListener getFaxStateActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetFaxState();
		}
	};
	
	private final ActionListener getFaxIdsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetFaxIds();
		}
	};
	
	private final ActionListener sendFaxActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doSendFax();
		}
	};
	
	private final ActionListener abortFiletransferActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doAbortFiletransfer();
		}
	};
	
	/// View Data
	////////////////////////////////////////////////////////////////
	
	private List<String> faxIds = new ArrayList<String>();
	
	private String faxNumber = Config.getValue("faxNumber");
	
	private String selectedFaxId = "";
	
	private String newFaxId = Config.getValue("faxId");
	
	private String calledNumber;
	
	private FaxState faxState;
	
	private int faxTransferProgress;
	
	private String faxTransferStatus = "";
	
	private int fileTransferProgress;
	
	private String fileTransferStatus = "";
	
	private boolean fileTransferAbortRequested;
	
	String textFileName = Config.getValue("textFile");
	
	String pdfFileName = Config.getValue("pdfFile");
	
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}
	
	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////
	
	public List<String> getFaxIds()
	{
		return faxIds;
	}
	
	public static final String PROPERTY_FAX_IDS = "faxIds";
	
	public void setFaxIds(List<String> faxIds)
	{
		List<String> old = this.faxIds;
		this.faxIds = faxIds;
		changeSupport.firePropertyChange(PROPERTY_FAX_IDS, old, faxIds);
	}
	
	public String getFaxNumber()
	{
		return faxNumber;
	}
	
	public static final String PROPERTY_FAX_NUMBER = "faxNumber";
	
	public void setFaxNumber(String faxNumber)
	{
		String old = this.faxNumber;
		this.faxNumber = faxNumber;
		changeSupport.firePropertyChange(PROPERTY_FAX_NUMBER, old, faxNumber);
	}
	
	public String getNewFaxId()
	{
		return newFaxId;
	}
	
	public static final String PROPERTY_NEW_FAX_ID = "newFaxId";
	
	public void setNewFaxId(String faxId)
	{
		String old = this.newFaxId;
		this.newFaxId = faxId;
		changeSupport.firePropertyChange(PROPERTY_NEW_FAX_ID, old, faxId);
	}
	
	public String getCalledNumber()
	{
		return calledNumber;
	}
	
	public static final String PROPERTY_CALLED_NUMBER = "calledNumber";
	
	public void setCalledNumber(String calledNumber)
	{
		String old = this.calledNumber;
		this.calledNumber = calledNumber;
		changeSupport.firePropertyChange(PROPERTY_CALLED_NUMBER, old, calledNumber);
	}
	
	public String getSelectedFaxId()
	{
		return selectedFaxId;
	}
	
	public static final String PROPERTY_SELECTED_FAX_ID = "selectedFaxId";
	
	public void setSelectedFaxId(String selectedCallId)
	{
		String old = this.selectedFaxId;
		this.selectedFaxId = selectedCallId;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_FAX_ID, old, selectedCallId);
	}
	
	public FaxState getFaxState()
	{
		return faxState;
	}
	
	public static final String PROPERTY_FAX_STATE = "faxState";
	
	public void setFaxState(FaxState faxState)
	{
		FaxState old = this.faxState;
		this.faxState = faxState;
		changeSupport.firePropertyChange(PROPERTY_FAX_STATE, old, faxState);
	}
	
	public FaxState[] getFaxStates()
	{
		return FaxState.values();
	}
	
	public int getFileTransferProgress()
	{
		return fileTransferProgress;
	}
	
	public static final String PROPERTY_FILE_TRANSFER_PROGRESS = "fileTransferProgress";
	
	public void setFileTransferProgress(int filetransferProgress)
	{
		int old = this.fileTransferProgress;
		this.fileTransferProgress = filetransferProgress;
		changeSupport.firePropertyChange(PROPERTY_FILE_TRANSFER_PROGRESS, old, filetransferProgress);
	}
	
	public String getFileTransferStatus()
	{
		return fileTransferStatus;
	}
	
	public static final String PROPERTY_FILE_TRANSFER_STATUS = "fileTransferStatus";
	
	public void setFileTransferStatus(String filetransferStatus)
	{
		String old = this.fileTransferStatus;
		this.fileTransferStatus = filetransferStatus;
		changeSupport.firePropertyChange(PROPERTY_FILE_TRANSFER_STATUS, old, filetransferStatus);
	}
	
	public boolean isFileTransferAbortRequested()
	{
		return fileTransferAbortRequested;
	}
	
	public static final String PROPERTY_FILE_TRANSFER_ABORT_REQUESTED = "fileTransferAbortRequested";
	
	public void setFileTransferAbortRequested(boolean filetransferAbortRequested)
	{
		boolean old = this.fileTransferAbortRequested;
		this.fileTransferAbortRequested = filetransferAbortRequested;
		changeSupport.firePropertyChange(PROPERTY_FILE_TRANSFER_ABORT_REQUESTED, old, filetransferAbortRequested);
	}
	
	public int getFaxTransferProgress()
	{
		return faxTransferProgress;
	}
	
	public static final String PROPERTY_FAX_TRANSFER_PROGRESS = "faxTransferProgress";
	
	public void setFaxTransferProgress(int faxTransferProgress)
	{
		int old = this.faxTransferProgress;
		this.faxTransferProgress = faxTransferProgress;
		changeSupport.firePropertyChange(PROPERTY_FAX_TRANSFER_PROGRESS, old, faxTransferProgress);
	}
	
	public String getFaxTransferStatus()
	{
		return faxTransferStatus;
	}
	
	public static final String PROPERTY_FAX_TRANSFER_STATUS = "faxTransferStatus";
	
	public void setFaxTransferStatus(String faxTransferStatus)
	{
		String old = this.faxTransferStatus;
		this.faxTransferStatus = faxTransferStatus;
		changeSupport.firePropertyChange(PROPERTY_FAX_TRANSFER_STATUS, old, faxTransferStatus);
	}
	
	public String getTextFileName()
	{
		return textFileName;
	}
	
	public static final String PROPERTY_TEXT_FILE_NAME = "textFileName";
	
	public void setTextFileName(String textFileName)
	{
		String old = this.textFileName;
		this.textFileName = textFileName;
		changeSupport.firePropertyChange(PROPERTY_TEXT_FILE_NAME, old, textFileName);
	}
	
	public String getPdfFileName()
	{
		return pdfFileName;
	}
	
	public static final String PROPERTY_PDF_FILE_NAME = "pdfFileName";
	
	public void setPdfFileName(String pdfFileName)
	{
		String old = this.pdfFileName;
		this.pdfFileName = pdfFileName;
		changeSupport.firePropertyChange(PROPERTY_PDF_FILE_NAME, old, pdfFileName);
	}
	
	public ActionListener getGetFaxStateActionListener()
	{
		return getFaxStateActionListener;
	}
	
	public ActionListener getGetFaxIdsActionListener()
	{
		return getFaxIdsActionListener;
	}
	
	public ActionListener getSendFaxActionListener()
	{
		return sendFaxActionListener;
	}
	
	public ActionListener getAbortFiletransferActionListener()
	{
		return abortFiletransferActionListener;
	}
	
}
