package de.starface.examples.uci30.swing.viewmodel;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.apache.commons.collections15.multimap.MultiHashMap;

import de.starface.integration.uci.java.v30.types.UciQueueCall;
import de.starface.integration.uci.java.v30.values.QueueCallState;
import de.starface.integration.uci.java.v30.values.UciQueueCallProperties;

public class QueueCallTableModel implements TableModel
{
	private QueueViewModel view;
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>(1);
	private int currentlySelectedGroupAccountId = 0;
	private static final UciQueueCallProperties[] queueCallColumnOrder;
	private static final SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
	private static final Date tooOldDate = new Date(10);
	static
	{
		queueCallColumnOrder = new UciQueueCallProperties[7];
		
		queueCallColumnOrder[0] = UciQueueCallProperties.callId;
		queueCallColumnOrder[1] = UciQueueCallProperties.callerName;
		queueCallColumnOrder[2] = UciQueueCallProperties.callerNumber;
		queueCallColumnOrder[3] = UciQueueCallProperties.state;
		queueCallColumnOrder[4] = UciQueueCallProperties.incommingTime;
		queueCallColumnOrder[5] = UciQueueCallProperties.connectedTime;
		queueCallColumnOrder[6] = UciQueueCallProperties.agents;
	}
	
	List<String> queueCalls = new LinkedList<String>();
	MultiHashMap<Integer, String> queue2CallMap = new MultiHashMap<Integer, String>();
	HashMap<String, UciQueueCall> callMap = new HashMap<String, UciQueueCall>();
	
	public QueueCallTableModel(QueueViewModel queueViewModel)
	{
		this.view = queueViewModel;
	}
	
	public void updateCall(UciQueueCall call)
	{
		boolean currentView = (call.getQueueAccountId() == currentlySelectedGroupAccountId);
		if (callMap.get(call.getCallId()) == null)
		{
			//new Call
			callMap.put(call.getCallId(), call);
			queue2CallMap.put(call.getQueueAccountId(), call.getCallId());
			queueCalls.add(call.getCallId());
		}
		else
		{
			if (call.getState() == QueueCallState.exited)
			{
				//call gone...
				callMap.remove(call.getCallId());
				Collection<String> calls = queue2CallMap.remove(call.getQueueAccountId());
				calls.remove(call.getCallId());
				queue2CallMap.putAll(call.getQueueAccountId(), calls);
				queueCalls.remove(call.getCallId());
			}
			else
			{
				//state-change, simply replace.
				callMap.remove(call.getCallId());
				callMap.put(call.getCallId(), call);
			}
		}
		if (currentView)
		{
			fireTableChanged(new TableModelEvent(this));
		}
	}
	
	public void setSelectedQueueId(int queueId)
	{
		currentlySelectedGroupAccountId = queueId;
		//calls aus der gruppe holen und liste neu initialisieren
		queueCalls.clear();
		Collection<String> calls = queue2CallMap.get(queueId);
		if (calls != null && calls.isEmpty())
		{
			queueCalls.addAll(calls);
		}
		fireTableChanged(new TableModelEvent(this));
	}
	
	//############################################
	// table stuff
	//############################################
	private void fireTableChanged(TableModelEvent e)
	{
		synchronized (listeners)
		{
			for (TableModelListener l : listeners)
			{
				l.tableChanged(e);
			}
		}
	}
	
	@Override
	public int getRowCount()
	{
		return queueCalls.size();
	}
	
	@Override
	public int getColumnCount()
	{
		return queueCallColumnOrder.length;
	}
	
	@Override
	public String getColumnName(int columnIndex)
	{
		return queueCallColumnOrder[columnIndex].name();
	}
	
	public Class<?> getColumnClass(int columnIndex)
	{
		switch (columnIndex)
		{
			case 4:
			case 5:
			case 6:
				return String.class;
			default:
				return queueCallColumnOrder[columnIndex].getType();
				
		}
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}
	
	@Override
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		UciQueueCall call = callMap.get(queueCalls.get(rowIndex));
		switch (columnIndex)
		{
			case 4:
				if (call.getIncommingTime() != null && call.getIncommingTime().after(tooOldDate))
				{
					return makeTime(call.getIncommingTime());
				}
				return "-";
			case 5:
				if (call.getConnectedTime() != null && call.getConnectedTime().after(tooOldDate))
				{
					return makeTime(call.getConnectedTime());
				}
				return "-";
			case 6:
				return makeAgentNames(call);
			default:
				return Utils.getProperty(queueCallColumnOrder[columnIndex], call);
		}
		
	}
	
	private String makeTime(Date connectedTime)
	{
		return dateFormat.format(connectedTime);
	}
	
	private String makeAgentNames(UciQueueCall uciQueueCall)
	{
		List<Integer> agents = uciQueueCall.getAgents();
		if (agents != null && !agents.isEmpty())
		{
			StringBuilder sb = new StringBuilder();
			sb.append(view.getAgentName(agents.get(0)));
			Iterator<Integer> it = agents.iterator();
			it.next(); //we already have the first agent;
			if (it.hasNext())
			{
				sb.append(", ");
				sb.append(view.getAgentName(it.next()));
			}
			return sb.toString();
		}
		
		return "";
	}
	
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			default:
		}
		
	}
	
	@Override
	public void addTableModelListener(TableModelListener l)
	{
		synchronized (listeners)
		{
			listeners.add(l);
		}
		
	}
	
	@Override
	public void removeTableModelListener(TableModelListener l)
	{
		synchronized (listeners)
		{
			listeners.remove(l);
		}
		
	}
	
	public void removeCallsOfQueue(Integer queueId)
	{
		
		Collection<String> calls2beRemoved = queue2CallMap.remove(queueId);
		for (String call : calls2beRemoved)
		{
			//call gone...
			callMap.remove(call);
			queueCalls.remove(call);
		}
	}
	
	public void init(List<UciQueueCall> calls)
	{
		for (UciQueueCall call : calls)
		{
			callMap.put(call.getCallId(), call);
			queue2CallMap.put(call.getQueueAccountId(), call.getCallId());
			queueCalls.add(call.getCallId());
		}
		fireTableChanged(new TableModelEvent(this));
		
	}
	
	public void updateCallId(String oldCallId, String newCallId)
	{
		UciQueueCall call = callMap.remove(oldCallId);
		//check, if this was a queueCall at all...
		if (call != null)
		{
			call.setCallId(newCallId);
			callMap.put(newCallId, call);
			Collection<String> calls = queue2CallMap.remove(call.getQueueAccountId());
			calls.remove(oldCallId);
			calls.add(newCallId);
			queue2CallMap.putAll(call.getQueueAccountId(), calls);
			queueCalls.remove(oldCallId);
			queueCalls.add(newCallId);
		}
	}
	
}
