/*
 * $Id: UciClient.java 296 2010-01-28 18:00:12Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2010-01-28 19:00:12 +0100 (Do, 28 Jan 2010) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 296 $
 */
package de.starface.integration.uci.java.v22.messages.requests;

import java.util.Date;
import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.UciServices;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v22.types.CallList;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v22.ucp.messages.requests.UcpCallListRequests;
import de.starface.integration.uci.java.v22.values.CallListEntryDirection;
import de.starface.integration.uci.java.v22.values.CallListEntryProperties;
import de.starface.integration.uci.java.v22.values.CallListEntryResult;
import de.starface.integration.uci.java.v22.values.GroupRestriction;
import de.starface.integration.uci.java.v22.values.OrderDirection;

/**
 * This interface contains the UCI Communication Call List Requests
 * 
 * @author tilmann
 */
@UciService(UcpCallListRequests.SERVICE_NAME)
@RpcInterface(UcpCallListRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciCallListRequests
{
	
	/**
	 * Retrieve a list of phone calls of the current user. The number of entries retrieved can be
	 * restricted by several parameters of this procedure call. The results are returned as
	 * {@link List} of {@link Map} containing the values for the keys defined in
	 * {@link CallListEntryProperties}.
	 * 
	 * @param startBefore
	 *        limit returned results to calls started before the given point of time
	 * @param startAfter
	 *        limit returned results to calls started after the given point of time
	 * @param directionRestriction
	 *        restrict returned results to calls of the given {@link CallListEntryDirection}. If
	 *        null is given no restriction is used.
	 * @param resultRestriction
	 *        restrict returned results to calls of the given {@link CallListEntryResult}. If null
	 *        is given no restriction is used.
	 * @param groupRestriction
	 *        restrict returned results to those conducted by a specified group. Possible choices
	 *        are the String {@link GroupRestriction#NON_GROUP} to return all calls that do not
	 *        belong to any group, the id of a group to restrict the list to the calls belonging to
	 *        that group and the empty String to return all calls without restriction.
	 * @param orderProperty
	 *        give one of the {@link CallListEntryProperties} to order results by the value of the
	 *        property. Allowed values are {@link CallListEntryProperties#startTime},
	 *        {@link CallListEntryProperties#duration}, {@link CallListEntryProperties#callerNumber}
	 *        and {@link CallListEntryProperties#calledNumber}. When null or an invalid property is
	 *        given the default {@link CallListEntryProperties#startTime} is used.
	 * @param orderDirection
	 *        give one of the {@link OrderDirection}s to define the ordering. If null is given the
	 *        default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link List} containing the result elements
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public CallList getCallList(Date startBefore, Date startAfter, CallListEntryDirection directionRestriction,
								CallListEntryResult resultRestriction, String groupRestriction,
								CallListEntryProperties orderProperty, OrderDirection orderDirection,
								int countOffset, int countLimit) throws UciException;
	
	/**
	 * Remove a call list entry from the list
	 * 
	 * @param callId
	 *        the id of the call list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteCallListEntry(String callId) throws UciException;
	
	/**
	 * Remove multiple call list entries from the list
	 * 
	 * @param callIds
	 *        the ids of the call list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteCallListEntries(List<String> callIds) throws UciException;
}
