/*
 * $Id: UcpFunctionKeyRequests.java 485 2011-01-31 14:52:54Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationFax.java $
 * 
 * $LastChangedDate: 2011-01-31 15:52:54 +0100 (Mo, 31 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 485 $
 */
package de.starface.integration.uci.java.v22.messages.requests;

import java.util.List;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.UciServices;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v22.types.CallInfo;
import de.starface.integration.uci.java.v22.types.ContactInfo;
import de.starface.integration.uci.java.v22.types.FunctionKey;
import de.starface.integration.uci.java.v22.types.Image;
import de.starface.integration.uci.java.v22.types.RedirectSetting;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v22.ucp.messages.requests.UcpFunctionKeyRequests;
import de.starface.integration.uci.java.v22.values.FunctionKeyProperties;
import de.starface.integration.uci.java.v22.values.FunctionKeyType;

/**
 * This interface defines the function key requests provided by the UCI server.
 * 
 * @author tilmann
 */
@UciService(UcpFunctionKeyRequests.SERVICE_NAME)
@RpcInterface(UcpFunctionKeyRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciFunctionKeyRequests
{
	/**
	 * This request allows the UCI client to retrieve the function keys of the current user.
	 * 
	 * @return A {@link List} of {@link FunctionKey} containing the function keys
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<FunctionKey> getFunctionKeys() throws UciException;
	
	/**
	 * This request allows the UCI client to virtually press a function key of the current user. A
	 * function key press usually leads to an action on the server depending on the
	 * {@link FunctionKeyType} of the key.
	 * 
	 * @param functionKeyId
	 *        the id of the function key to be pressed.
	 * @param parameters
	 *        additional parameters used to modify the behavior of the action to be activated upon
	 *        the key press. The meaning of the parameters depends on the function key to be
	 *        pressed. It is documented together with the {@link FunctionKeyType}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void pressKey(String functionKeyId, Object... parameters) throws UciException;
	
	/**
	 * This request allows the UCI client to query which redirect always settings have been
	 * activated by the user or group the key of type {@link FunctionKeyType#BUSY_LAMP_FIELD} shows
	 * the status of.
	 * 
	 * @param functionKeyId
	 *        the id of the function key to obtain the settings for
	 * @return a list with the redirect settings
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<RedirectSetting> getRedirectSettingsForKey(String functionKeyId)
		throws UciException;
	
	/**
	 * This request allows the UCI client to query the contact information of the user or group the
	 * key of type {@link FunctionKeyType#BUSY_LAMP_FIELD} shows the status of.
	 * 
	 * @param functionKeyId
	 *        the id of the function key to obtain the contact information for
	 * @return the contact information of the user or group
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public ContactInfo getContactInfoForKey(String functionKeyId) throws UciException;
	
	/**
	 * This request allows the UCI client to query information about the calls currently being held
	 * or received by the user or group the key of type {@link FunctionKeyType#BUSY_LAMP_FIELD}
	 * shows the status of.
	 * 
	 * @param functionKeyId
	 *        the id of the function key to obtain the call information for
	 * @return the information about the calls in a list
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public List<CallInfo> getCallInfoForKey(String functionKeyId) throws UciException;
	
	/**
	 * This request allows the UCI client to query the image of the key. A key has an image if its
	 * property {@link FunctionKeyProperties#imageHash} is not empty. Currently only keys of type
	 * {@link FunctionKeyType#BUSY_LAMP_FIELD} that show the status of a user can have an image
	 * which then is the avatar image of the user.
	 * 
	 * @param functionKeyId
	 *        the id of the function key to obtain the image for
	 * @return the avatar image of the user
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Image getImageForKey(String functionKeyId) throws UciException;
	
}
