package de.starface.integration.uci.java.v22.ucp.messages.requests;

import java.util.List;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpNotConnectedException;

@RpcInterface(UcpServiceRequests.SERVICE_NAME)
@UciService(UcpServiceRequests.SERVICE_NAME)
public interface UcpServiceRequests
{
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.requests.service";
	
	/**
	 * The client logged in at the server may subscribe to events provided by the server. To do so
	 * it provides a service name depending on the events it wants to receive. The server will only
	 * send the events requested in this way. The client is not required to subscribe to any events
	 * but this method to get informed of server state is preferred over polling.
	 * 
	 * @param serviceName
	 *        a String denoting the service name of the events the client wants to receive
	 * @return <code>true</code> if the registration was successful or <code>false</code> if the
	 *         requested events are not provided by the server
	 * @throws UciException
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public boolean subscribeEvents(String serviceName) throws UciException;
	
	/**
	 * The client logged in may also unsubscribe any events it has previously subscribed to. To do
	 * so it will send this request providing a service names to unsubscribe from.
	 * 
	 * @param serviceName
	 *        a String denoting the service name of the events to unsubscribe from
	 * @throws UciException
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public void unsubscribeEvents(String serviceName) throws UciException;
	
	/**
	 * This request can be used by the client to detect if events it wants to use are actually
	 * provided by the server. To do so it sends this request providing a service name of the events
	 * intended to be used.
	 * 
	 * @param serviceName
	 *        a String denoting the service name of the events to query for
	 * @return <code>true</code> if the given events are supported by the server or
	 *         <code>false</code> if not supported.
	 * @throws UciException
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public boolean providesEvents(String serviceName) throws UciException;
	
	/**
	 * This request can be used to query the server for the service names of all supported events.
	 * 
	 * @return a List of String containing the service names of supported events
	 * @throws UciException
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getProvidedEvents() throws UciException;
	
	/**
	 * This request can be used by the client to detect if another requests it wants to use are
	 * actually provided by the server. To do so it sends this request providing a service name of
	 * the requests intended to be used.
	 * 
	 * @param serviceName
	 *        a String denoting the service name of the requests to query for
	 * @return <code>true</code> if the given requests are supported by the server or
	 *         <code>false</code> if not supported.
	 * @throws UciException
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public boolean providesRequests(String serviceName) throws UciException;
	
	/**
	 * This request can be used to query the server for the service names of all supported requests.
	 * 
	 * @return a List of String containing the service names of supported requests
	 * @throws UciException
	 *         if the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getProvidedRequests() throws UciException;
}
