/*
 * $Id: UcpVoicemailListRequests.java 650 2011-11-29 18:33:32Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationVoicemailList.java $
 * 
 * $LastChangedDate: 2011-11-29 19:33:32 +0100 (Di, 29. Nov 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 650 $
 */
package de.starface.integration.uci.java.v22.ucp.messages.requests;

import java.util.Date;
import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.services.filetransfer.common.RpcFileDownload;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.exceptions.UciException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v22.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v22.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v22.values.MailboxProperties;
import de.starface.integration.uci.java.v22.values.OrderDirection;
import de.starface.integration.uci.java.v22.values.VoicemailListEntryFolder;
import de.starface.integration.uci.java.v22.values.VoicemailListEntryProperties;
import de.starface.integration.uci.java.v22.values.VoicemailListProperties;

/**
 * 
 * This interface defines the voicemail list requests provided by the UCI server.
 * 
 * @author tilmann
 * 
 */
@RpcInterface(UcpVoicemailListRequests.SERVICE_NAME)
@UciService(UcpVoicemailListRequests.SERVICE_NAME)
public interface UcpVoicemailListRequests
{
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.requests.voicemailList";
	
	/**
	 * With this request the UCI client can retrieve a list of voicemails of the current user. The
	 * number of entries retrieved can be restricted by several parameters of this procedure call.
	 * The results are returned as {@link Map} containing the values for the keys defined in
	 * {@link VoicemailListProperties}.
	 * 
	 * @param startBefore
	 *        limit returned results to voicemails started before the given point of time
	 * @param startAfter
	 *        limit returned results to voicemails started after the given point of time
	 * @param mailboxRestriction
	 *        restrict returned results to voicemails in the mailbox with the given id. Available
	 *        mailboxes for the current user may be retrieved with a call to {@link #getMailboxes()}
	 *        . If an empty String is given no restriction is used.
	 * @param folderRestriction
	 *        restrict returned results to voicemails in the given folder defined in
	 *        {@link VoicemailListEntryFolder}. If an empty String is given no restriction is used.
	 * @param orderProperty
	 *        give the name of one of the {@link VoicemailListEntryProperties} to order results by
	 *        the value of the property. Allowed values are
	 *        {@link VoicemailListEntryProperties#startTime},
	 *        {@link VoicemailListEntryProperties#duration},
	 *        {@link VoicemailListEntryProperties#callerNumber} and
	 *        {@link VoicemailListEntryProperties#calledNumber}. When an empty String or invalid
	 *        property name is given the default {@link VoicemailListEntryProperties#startTime} is
	 *        used.
	 * @param orderDirection
	 *        give the name of one of the {@link OrderDirection}s to define the ordering. If not set
	 *        the default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link Map} containing the results with the properties defined in
	 *         {@link VoicemailListProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getVoicemailList(Date startBefore, Date startAfter, String mailboxRestriction,
												String folderRestriction, String orderProperty,
												String orderDirection, int countOffset, int countLimit)
		throws UciException;
	
	/**
	 * This request allows the UCI client to remove a voicemail list entry from the list
	 * 
	 * @param voicemailId
	 *        the id of the voicemail list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteVoicemailListEntry(String voicemailId) throws UciException;
	
	/**
	 * This request allows the UCI client to remove multiple voicemail list entries from the list
	 * 
	 * @param voicemailIds
	 *        the ids of the voicemail list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteVoicemailListEntries(List<String> voicemailIds) throws UciException;
	
	/**
	 * This request moves a voicemail list entry to another folder
	 * 
	 * @param voicemailId
	 *        the id of the voicemail list entry to move
	 * @param destinationFolder
	 *        the name of the folder to move the entry to as defined in
	 *        {@link VoicemailListEntryFolder}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void moveVoicemailListEntry(String voicemailId, String destinationFolder) throws UciException;
	
	/**
	 * This request moves multiple voicemail list entries to another folder
	 * 
	 * @param voicemailIds
	 *        the ids of the voicemail list entries to move
	 * @param destinationFolder
	 *        the name of the folder to move the entry to as defined in
	 *        {@link VoicemailListEntryFolder}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void moveVoicemailListEntries(List<String> voicemailIds, String destinationFolder) throws UciException;
	
	/**
	 * This request creates and returns a temporary URL that can be used to download the voicemail
	 * file for the next 15 minutes.
	 * 
	 * @param voicemailId
	 *        the id of the voicemail list entry to retrieve the file for
	 * @return The download URL as {@link String}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getVoicemailFileUrl(String voicemailId) throws UciException;
	
	/**
	 * Prepares the download of the voicemail for the given voicemail id. After a calling this
	 * request the file can be downloaded using the interface {@link VoicemailFileDownload} and the
	 * returned fileId.
	 * 
	 * @param voicemailId
	 *        the id of the voicemail list entry to retrieve the file for
	 * @return the fileId that can be used to download the file from STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getVoicemailFile(String voicemailId) throws UciException;
	
	/**
	 * This interface defines the RPC methods that can be used to download the voicemail files
	 */
	@RpcInterface(SERVICE_NAME + ".getVoicemailFile.rpcFileDownload")
	public interface VoicemailFileDownload extends RpcFileDownload
	{
		// Just mapping the procedures from the super interface with the correct service name
	}
	
	/**
	 * This request allows the UCI client to retrieve a list with all mailboxes of the current user
	 * 
	 * @return a {@link List} with one {@link Map} for each mailbox containing the properties as
	 *         defined in {@link MailboxProperties}.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public List<Map<String, String>> getMailboxes() throws UciException;
	
	/**
	 * With this request the UCI client can initiate a phone call to the given phone number that
	 * when answered is connected to one of the current users voicemail box
	 * 
	 * @param mailboxId
	 *        the id of the mailbox to connect to
	 * @param originatingNumber
	 *        the phone number to be called
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callMailboxWithNumber(String mailboxId, String originatingNumber) throws UciException;
	
	/**
	 * With this request the UCI client can initiate a phone call to the given voicemail box using
	 * the given phone
	 * 
	 * @param mailboxId
	 *        the id of the mailbox to connect to
	 * @param phoneId
	 *        the id of the phone used to initiate the call with as returned by
	 *        {@link UcpPhoneRequests#getPhoneIds()} If not set the primary phone will be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callMailboxWithPhone(String mailboxId, String phoneId) throws UciException;
	
	/**
	 * With this request the UCI client can initiate a phone call to the given phone number that
	 * when answered plays one of the current users voicemails.
	 * 
	 * @param voicemailId
	 *        the id of the voicemail to be played
	 * @param originatingNumber
	 *        the phone number to be called
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callVoicemailWithNumber(String voicemailId, String originatingNumber) throws UciException;
	
	/**
	 * With this request the UCI client can initiate a call with the given phone that plays one of
	 * the current users voicemails.
	 * 
	 * @param voicemailId
	 *        the id of the voicemail to be played
	 * @param phoneId
	 *        the id of the phone used to initiate the call with as returned by
	 *        {@link UcpPhoneRequests#getPhoneIds()} If not set the primary phone will be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callVoicemailWithPhone(String voicemailId, String phoneId) throws UciException;
}
