/*
 * $Id: UciClient.java 296 2010-01-28 18:00:12Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2010-01-28 19:00:12 +0100 (Do, 28 Jan 2010) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 296 $
 */
package de.starface.integration.uci.java.v30.messages.requests;

import java.io.File;
import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.com.rpc.client.RpcRequestCaller;
import de.starface.com.rpc.services.filetransfer.FileTransferEventListener;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.messages.events.UciFaxEvents;
import de.starface.integration.uci.java.v30.types.Fax;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpFaxRequests;
import de.starface.integration.uci.java.v30.values.FaxProperties;

/**
 * This interface contains the UCI Communication Fax Requests
 * 
 * @author tilmann
 */
@UciService(UcpFaxRequests.SERVICE_NAME)
@RpcInterface(UcpFaxRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION, automaticFileTransfer = true)
public interface UciFaxRequests
{
	/**
	 * The UCI client may call getFaxIds to have the server send a response with a list of fax ids
	 * of the faxes being sent by the user.
	 * 
	 * @return the UUIDs assigned to the existing faxes
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getFaxIds() throws UciException;
	
	/**
	 * The getFaxState message is used to query STARFACE for the state of an identified fax request.
	 * 
	 * @param faxId
	 *        the UUID assigned to the fax request
	 * @return the different properties of the fax in a {@link Map}. The constants of
	 *         {@link FaxProperties} are used as keys in the map. If the fax is not known an empty
	 *         dictionary is returned.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Fax getFaxState(String faxId) throws UciException;
	
	/**
	 * This procedure is called to initiate a fax sending process. This includes that both provided
	 * files are completely transferred to the server. A successful send fax request returns after
	 * the fax sending process has been started but before the fax has actually been sent to the
	 * destination. To obtain information about the ongoing fax sending process a client may
	 * subscribe to the {@link UciFaxEvents}.
	 * 
	 * @param faxNumber
	 *        the phone number to dial
	 * @param faxId
	 *        The UUID to be assigned to the fax request. If not set STARFACE will create one.
	 * @param textFile
	 *        the text file containing the fax as plain text. This file is optional.
	 * @param pdfFile
	 *        the PDF file containing the PDF to be sent as fax.
	 * @return an empty {@link String} if the fax request was rejected by the server or the UUID
	 *         assigned to the fax request otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String sendFax(String faxNumber, String faxId, File textFile, File pdfFile) throws UciException;
	
	/**
	 * This procedure is called to initiate a fax sending process. The method is invoked
	 * asynchronously and returns before files are transfered to the server and before the fax
	 * sending process has been started. To get informed about the file transfers a
	 * {@link FileTransferEventListener} can be registered with the {@link RpcRequestCaller} used to
	 * make the call. To obtain information about the ongoing fax sending process a client may
	 * subscribe to the {@link UciFaxEvents}.
	 * 
	 * @param faxNumber
	 *        the phone number to dial
	 * @param faxId
	 *        The UUID to be assigned to the fax request. If not set STARFACE will create one.
	 * @param textFile
	 *        the text file containing the fax as plain text. This file is optional.
	 * @param pdfFile
	 *        the PDF file containing the PDF to be sent as fax.
	 * 
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	@RpcAsynchronous
	public void sendFaxAsync(String faxNumber, String faxId, File textFile, File pdfFile) throws UciException;
	
}
