/*
 * $Id: UcpCallRequests.java 483 2011-01-26 18:04:04Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCall.java $
 * 
 * $LastChangedDate: 2011-01-26 19:04:04 +0100 (Mi, 26 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 483 $
 */
package de.starface.integration.uci.java.v30.messages.requests;

import java.util.List;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.PhoneNumber;
import de.starface.integration.uci.java.v30.types.SoftPhoneConfig;
import de.starface.integration.uci.java.v30.types.SoftPhoneSetting;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpPhoneRequests;

/**
 * This interface defines the phone requests provided by the UCI server.
 * 
 * @author tilmann
 */
@UciService(UcpPhoneRequests.SERVICE_NAME)
@RpcInterface(UcpPhoneRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UciPhoneRequests
{
	
	/**
	 * The UCI client may call getPhoneIds to have the server send a response with a list of
	 * telephone ids of the phones assigned to the user.
	 * 
	 * @return a list of strings representing the ids of the users telephones
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getPhoneIds() throws UciException;
	
	/**
	 * This request allows the UCI client to retrieve the id of the primary phone of the user. This
	 * is the phone that is used to place an outgoing call if no explicit phone is selected during
	 * the place call request.
	 * 
	 * @return the id of the primary phone
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public String getPrimaryPhoneId() throws UciException;
	
	/**
	 * This request allows the UCI client to set the primary phone of the user. This is the phone
	 * that is used to place an outgoing call if no explicit phone is selected during the place call
	 * request.
	 * 
	 * @param phoneId
	 *        the id of the primary phone to be set. This can be any id obtained by
	 *        {@link #getPhoneIds()}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setPrimaryPhoneId(String phoneId) throws UciException;
	
	/**
	 * This request allows the UCI client to retrieve the phone number that is signaled to the other
	 * party during a call on an outgoing line.
	 * 
	 * @return the id of the phone number to be set as display number or an empty {@link String} if
	 *         the display number is being suppressed. Possible values can be retrieved using
	 *         {@link #getAvailableDisplayNumbers()} request.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public String getDisplayNumberId() throws UciException;
	
	/**
	 * This request allows the UCI client to set the phone number that is signaled to the other
	 * party during a call on an outgoing line.
	 * 
	 * @param phoneNumberId
	 *        the id of the phone number to be set as display number or an empty {@link String} if
	 *        the display number should be suppressed. Possible values can be retrieved using
	 *        {@link #getAvailableDisplayNumbers()} request.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setDisplayNumberId(String phoneNumberId) throws UciException;
	
	/**
	 * This request allows the UCI client to obtain a list of phone number that are available to be
	 * set as display number for the user.
	 * 
	 * @return a {@link List} of {@link PhoneNumber} containing the phone numbers
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<PhoneNumber> getAvailableDisplayNumbers() throws UciException;
	
	/**
	 * This request allows the UCI client to check if the call waiting indication is used for
	 * simultaneous phone calls.
	 * 
	 * @return {@code true} if call waiting indication is active, {@code false} otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public boolean getCallWaitingIndication() throws UciException;
	
	/**
	 * This request allows the UCI client to configure the usage of call waiting indication for
	 * simultaneous phone calls.
	 * 
	 * @param callWaitingIndication
	 *        a flag indicating if call waiting indication should be activated or deactivated.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public void setCallWaitingIndication(boolean callWaitingIndication) throws UciException;
	
	/**
	 * This request allows the UCI client to check wether a SipPhone with the softPhoneId of the
	 * given SoftPhoneSetting is currently registered for the user.
	 * 
	 * @param softPhoneSetting
	 *        the {@link SoftPhoneSetting} that identifies the requested SoftPhone by its
	 *        softPhoneId, other fields of the {@link SoftPhoneSetting} will be ignored
	 * @return true if the SipPhone is in use by another SoftPhone instance, otherwise false
	 */
	public boolean isSipPhoneInUse(SoftPhoneSetting softPhoneSetting) throws UciException;
	
	/**
	 * This request allows the UCI client to get SIP credentials for a SoftPhone. If the requested
	 * SoftPhone has not already been registered a new SoftPhone will be autoprovisioned and
	 * assigned to the user.
	 * 
	 * @param softPhoneSetting
	 *        the {@link SoftPhoneSetting} that identifies the requested SoftPhone
	 * @return the SoftPhoneConfig holding SIP credentials for the requested SoftPhone
	 */
	public SoftPhoneConfig registerSipPhone(SoftPhoneSetting softPhoneSetting) throws UciException;
}
