package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.QueueAgentStateProperties;
import de.starface.integration.uci.java.v30.values.QueueSettingProperties;
import de.starface.integration.uci.java.v30.values.UciQueueCallProperties;

@UciService(UCPQueueRequests.SERVICE_NAME)
@RpcInterface(UCPQueueRequests.SERVICE_NAME)
public interface UCPQueueRequests
{
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.queue";
	
	/**
	 * This request allows the UCI client to query the queues.
	 * 
	 * @return a {@link List} with a {@link Map} for each queue containing the values for the
	 *         properties defined in {@link QueueSettingProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public abstract List<Map<String, Object>> getQueueSettings() throws UciException;
	
	/**
	 * Returns the Settings of a queue as defined in {@link QueueSettingProperties}
	 * 
	 * @param queueAccountId
	 *        the accountId of the specific queue
	 * @return the settings of a queue
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public abstract Map<String, Object> getQueueSetting(int queueAccountId) throws UciException;
	
	/**
	 * Returns a List of QueueAgents as defined in {@link QueueAgentStateProperties} for the given
	 * queue
	 * 
	 * @param queueAccountId
	 *        the accountId of the specific queue
	 * @return a list of queueAgents
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ( {@link UcpNotConnectedException}).
	 */
	public abstract List<Map<String, Object>> getQueueAgents(int queueAccountId) throws UciException;
	
	/**
	 * Returns a List of ongoing queue calls as defined in {@link UciQueueCallProperties}<br>
	 * Use this <b>for initialization only</b>. The interface will update any changes by events that
	 * contain all the necessary information. <i>Continuous polling will lead to <b>bad system
	 * performance!</i></b>
	 * 
	 * @param queueAccountId
	 *        the accountid of the specific queue
	 * @return a list of queueCalls
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ( {@link UcpNotConnectedException}).
	 */
	public List<Map<String, Object>> getCallsForQueue(int queueAccountId) throws UciException;
	
}
