/*
 * $Id: UcpCallRequests.java 950 2014-06-20 11:35:16Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCall.java $
 * 
 * $LastChangedDate: 2014-06-20 13:35:16 +0200 (Fr, 20. Jun 2014) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 950 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.types.Image;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.CallProperties;
import de.starface.integration.uci.java.v30.values.CallState;

/**
 * This interface defines the call requests provided by the UCI server.
 * 
 * @author tilmann
 */
@UciService(UcpCallRequests.SERVICE_NAME)
@RpcInterface(UcpCallRequests.SERVICE_NAME)
public interface UcpCallRequests
{
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.call";
	
	/**
	 * The UCI client may call getCallIds to have the server send a response with a list of call ids
	 * of the call being held by the user.
	 * 
	 * @return the UUIDs assigned to the existing phone calls
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request is not allowed to the user ({@link UciIllegalAccessException}) or if
	 *         the request is placed while the UCI client is not connected (
	 *         {@link UcpNotConnectedException}).
	 */
	public List<String> getCallIds() throws UciException;
	
	/**
	 * The getCallState request provides a number of properties about an identified call. This
	 * includes the state of the call. The parameters callerNumber, callerName, calledNumber and
	 * calledName provide information about the parties involved in the call. If a name or number is
	 * not known to the UCI server an empty string is committed. It is possible that information
	 * about the caller or the called person is available on a later call.
	 * 
	 * @param callId
	 *        the UUID assigned to the phone call
	 * @return the different properties of the call in a {@link Map}. The constants of
	 *         <code>{@link CallProperties}</code> are used as keys in the {@link Map}. If the call
	 *         is not known an empty map is returned.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getCallState(String callId) throws UciException;
	
	/**
	 * This request is called to initiate a phone call for the user the same way as when using the
	 * STARFACE call manager. The phone number to call and phone to use can be selected using the
	 * request parameters. In UCI phone calls are identified using UUIDs (Universally Unique
	 * Identifier). The id for the call to be placed can either be given by the client or generated
	 * by STARFACE. The server may reject the request to place the call due to another place call
	 * request being in progress. If the result of the message is a valid UUID the call request is
	 * accepted and the call is in the state {@link CallState#OUTGOING}.
	 * 
	 * @param destinationNumber
	 *        The phone number to dial
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @param callId
	 *        The UUID to be assigned to the phone call. If not set STARFACE will create one.
	 * @return an empty string if the place call request was rejected by the server or the UUID
	 *         assigned to the call otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String placeCallWithPhone(String destinationNumber, String phoneId, String callId) throws UciException;
	
	/**
	 * This request is called to initiate a phone call for the user. First a call is made to the
	 * originating number and if the call is answered a call to the destination number is
	 * established and connected with the originating call. In UCI phone calls are identified using
	 * UUIDs (Universally Unique Identifier). The id for the call to be placed can either be given
	 * by the client or generated by STARFACE. The server may reject the request to place the call
	 * due to another place call request being in progress. If the result of the message is a valid
	 * UUID the call request is accepted and the call is in the state {@link CallState#OUTGOING}.
	 * 
	 * @deprecated WARNING: Initiating a call this way results in a phone call that does not provide
	 *             all of the UCI 3.0 Call Events denoted by the UCI 3.0 documentation. Some events
	 *             may be missing or out of order.
	 * 
	 * @param destinationNumber
	 *        The phone number to dial
	 * @param originatingNumber
	 *        The phone number to dial first that will be used to originate the call. be used.
	 * @param callId
	 *        The UUID to be assigned to the phone call. If not set STARFACE will create one.
	 * @return an empty string if the place call request was rejected by the server or the UUID
	 *         assigned to the call otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 * 
	 */
	public String placeCallWithNumber(String destinationNumber, String originatingNumber, String callId)
		throws UciException;
	
	/**
	 * This request tells the UCI server to hang up the call identified by the given UUID.
	 * 
	 * @param callId
	 *        the UUID of the call to be hung up
	 * @return <code>true</code> if the attempt to hang up the call was successful,
	 *         <code>false</code> otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public boolean hangupCall(String callId) throws UciException;
	
	/**
	 * This request tells the UCI server to hold the call identified by the given UUID.
	 * 
	 * @param callId
	 *        the UUID of the call to be held
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void hold(String callId) throws UciException;
	
	/**
	 * This request tells the UCI server to resume the call identified by the given UUID and the
	 * given Phone.
	 * 
	 * @param callId
	 *        the UUID of the call to be held
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void resume(String callId, String phoneId) throws UciException;
	
	/**
	 * This request tells the UCI server to create a conference with calls identified by the given
	 * UUIDs and the given Phone.
	 * 
	 * @param callIds
	 *        the UUID of the calls to be redirected
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void conference(List<String> callIds, String phoneId) throws UciException;
	
	/**
	 * This request tells the UCI server to forward a call identified by the given UUID to the given
	 * Extension.
	 * 
	 * @param callId
	 *        the UUID of the call to be forwarded.
	 * @param extension
	 *        The extension to be dialed.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void forward(String callId, String extension) throws UciException;
	
	/**
	 * This request tells the UCI server to transfer a call identified by the given UUID to the
	 * given parked UUID.
	 * 
	 * @param callId
	 *        the UUID of the call to be transfered.
	 * @param callIdParked
	 *        the parked call.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void transfer(String callId, String callIdParked) throws UciException;
	
	/**
	 * This request tells the UCI server to swith the phone of the given call identified by the
	 * given UUID.
	 * 
	 * @param callId
	 *        the UUID of the call to be transfered.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void switchPhone(String callId) throws UciException;
	
	/**
	 * This request tells the UCI server to monitor the given call identified by the given UUID.
	 * 
	 * @param callId
	 *        the UUID of the call to be transfered.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void monitor(String callId) throws UciException;
	
	/**
	 * This request tells the UCI server to park and give an orbit number to the given call
	 * identified by the given UUID. If an call is in the given orbit number you will be connected
	 * to the parked member
	 * 
	 * @param callId
	 *        the UUID of the call to be transfered.
	 * @param parkAndOrbitNumber
	 *        the number of the parkorbit
	 * @param phone
	 *        The id of the telephone used to make the resume of the parked orbit. If not set the
	 *        primary phone will be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void parkAndOrbit(String callId, String parkAndOrbitNumber, String phone) throws UciException;
	
	/**
	 * This request tells the UCI server to call the users voicemailbox.
	 * 
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void callVoicemailBox(String phoneId) throws UciException;
	
	/**
	 * This request tells the UCI server to grab the call from the linked voicemailbox.
	 * 
	 * @param callId
	 *        the UUID of the call to be grabbed.
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void grabVoicemailBoxCallerWithPhone(String callId, String phoneId) throws UciException;
	
	/**
	 * This request tells the UCI server the call to the voicemailbox.
	 * 
	 * @param callId
	 *        the UUID of the call to be grabbed.
	 * @param mailboxId
	 *        The id of the mailbox used to record the call.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void transferToVoicemailBox(String callId, String mailboxId) throws UciException;
	
	/**
	 * This request tells the UCI server to send current messageWaitingEvents for all mailboxes of
	 * the account.
	 * 
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void getCurrentMessageWaitingState() throws UciException;
	
	/**
	 * This request tells the UCI server to send DTMF sounds on that call.
	 * 
	 * @param callId
	 *        the UUID of the call.
	 * 
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void playDtmf(String callId, String dtmfDigits) throws UciException;
	
	/**
	 * This request tells the UCI server to return the avatar image of an image hash.
	 * 
	 * @param imageHash
	 *        the hash value of the image.
	 * 
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Image getImageOfHash(String imageHash) throws UciException;
	
	/**
	 * This request is called to initiate a phone consultation call for the user the same way as
	 * when using the STARFACE call manager. The phone number to call and phone to use can be
	 * selected using the request parameters. In UCI phone calls are identified using UUIDs
	 * (Universally Unique Identifier). The id for the call to be placed can either be given by the
	 * client or generated by STARFACE. The server may reject the request to place the call due to
	 * another place call request being in progress. If the result of the message is a valid UUID
	 * the call request is accepted and the call is in the state {@link CallState#OUTGOING}.
	 * 
	 * @param destinationNumber
	 *        The phone number to dial
	 * @param phoneId
	 *        The id of the telephone used to make the phone call. If not set the primary phone will
	 *        be used.
	 * @param callId
	 *        The UUID to be assigned to the phone call. If not set STARFACE will create one.
	 * @param callIdReference
	 *        The UUID of the referenced Call.
	 * @return an empty string if the place call request was rejected by the server or the UUID
	 *         assigned to the call otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String placeConsultationCallWithPhone(String destinationNumber, String phoneId, String callId,
												 String callIdReference) throws UciException;
	
	/**
	 * This request is called to transfer a consultation call for the user the same way as when
	 * using the STARFACE call manager. The phone number to call and phone to use can be selected
	 * using the request parameters. In UCI phone calls are identified using UUIDs (Universally
	 * Unique Identifier). The id for the call to be placed can either be given by the client or
	 * generated by STARFACE. The server may reject the request to place the call due to another
	 * place call request being in progress.
	 * 
	 * @param callId
	 *        The UUID to be transfered.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void transferConsultationCall(String callId) throws UciException;
	
	/**
	 * This request is called to transfer a consultation call to conference for the user the same
	 * way as when using the STARFACE call manager. The phone number to call and phone to use can be
	 * selected using the request parameters. In UCI phone calls are identified using UUIDs
	 * (Universally Unique Identifier). The id for the call to be placed can either be given by the
	 * client or generated by STARFACE. The server may reject the request to place the call due to
	 * another place call request being in progress.
	 * 
	 * @param callId
	 *        The UUID to be transfered.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void transferToConferenceConsultationCall(String callId) throws UciException;
}
