/*
 * $Id: UcpFaxRequests.java 847 2013-11-20 11:18:26Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationFax.java $
 * 
 * $LastChangedDate: 2013-11-20 12:18:26 +0100 (Mi, 20. Nov 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 847 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.services.filetransfer.common.RpcFileUpload;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.FaxProperties;

/**
 * 
 * This interface defines the fax requests provided by the UCI server.
 * 
 * @author tilmann
 * 
 */
@RpcInterface(UcpFaxRequests.SERVICE_NAME)
@UciService(UcpFaxRequests.SERVICE_NAME)
public interface UcpFaxRequests
{
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.fax";
	
	/**
	 * The UCI client may call getFaxIds to have the server send a response with a list of fax ids
	 * of the faxes currently being sent by the user.
	 * 
	 * @return the UUIDs assigned to the existing faxes
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public List<String> getFaxIds() throws UciException;
	
	/**
	 * The getFaxState request is used to query the UCI server for the state of an identified fax
	 * request.
	 * 
	 * @param faxId
	 *        the UUID assigned to the fax request
	 * @return the different properties of the fax in a {@link Map}. The constants of
	 *         {@link FaxProperties} are used as keys in the map. If the fax is not known to
	 *         STARFACE an empty {@link Map} is returned.
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getFaxState(String faxId) throws UciException;
	
	/**
	 * This request initiates the fax sending process. Before this can be done a PDF file to be sent
	 * as fax must and a text file containing the fax text may be uploaded to the server using the
	 * {@link UcpFaxRequests$SendFaxFileUpload} RPC interface. In the next step sendFax can be
	 * invoked and the destination number of the fax must be given. In UCI faxes are identified
	 * using UUIDs (Universally Unique Identifier). The id for the fax to be sent can either be
	 * given by the client or generated by STARFACE. In the case of a successful response (denoted
	 * by a non empty returned value), the fax sending process has been started. However the fax has
	 * not actually been sent to the destination yet.
	 * 
	 * @param faxNumber
	 *        the phone number to send the fax to
	 * @param faxId
	 *        The UUID to be assigned to the fax request. If not set STARFACE will create one.
	 * @param textFileId
	 *        the id of the file transfer used to upload the text file. If no text file has been
	 *        uploaded the empty string must be given.
	 * @param pdfFileId
	 *        the id of the file transfer used to upload the PDF file.
	 * @return an empty string if the fax request was rejected by the server or the UUID assigned to
	 *         the fax request otherwise
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String sendFax(String faxNumber, String faxId, String textFileId, String pdfFileId) throws UciException;
	
	/**
	 * This interface defines the RPC methods that can be used to upload the fax files
	 */
	@RpcInterface(SERVICE_NAME + ".sendFax.rpcFileUpload")
	public interface SendFaxFileUpload extends RpcFileUpload
	{
		// Just mapping the procedures from the super interface with the correct service name
	}
	
}
