/*
 * $Id: UcpVoicemailListRequests.java 847 2013-11-20 11:18:26Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationVoicemailList.java $
 * 
 * $LastChangedDate: 2013-11-20 12:18:26 +0100 (Mi, 20. Nov 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 847 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.Date;
import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.services.filetransfer.common.RpcFileDownload;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.MonitorListEntryProperties;
import de.starface.integration.uci.java.v30.values.MonitorListProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;

/**
 * 
 * This interface defines the monitor list requests provided by the UCI server.
 * 
 * @author tilmann
 * 
 */
@RpcInterface(UcpMonitorListRequests.SERVICE_NAME)
@UciService(UcpMonitorListRequests.SERVICE_NAME)
public interface UcpMonitorListRequests
{
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.monitorList";
	
	/**
	 * With this request the UCI client can retrieve a list of monitors of the current user. The
	 * number of entries retrieved can be restricted by several parameters of this procedure call.
	 * The results are returned as {@link Map} containing the values for the keys defined in
	 * {@link MonitorListProperties}.
	 * 
	 * @param startBefore
	 *        limit returned results to monitors started before the given point of time
	 * @param startAfter
	 *        limit returned results to monitors started after the given point of time
	 * @param orderProperty
	 *        give the name of one of the {@link MonitorListEntryProperties} to order results by the
	 *        value of the property. Allowed values are
	 *        {@link MonitorListEntryProperties#monitortimestart} ,
	 *        {@link MonitorListEntryProperties#duration},
	 *        {@link MonitorListEntryProperties#callercallerid} and
	 *        {@link MonitorListEntryProperties#calledcallerid}. When an empty String or invalid
	 *        property name is given the default {@link MonitorListEntryProperties#monitortimestart}
	 *        is used.
	 * @param orderDirection
	 *        give the name of one of the {@link OrderDirection}s to define the ordering. If not set
	 *        the default order direction of the property will be used
	 * @param countOffset
	 *        the offset of the first returned value in the result list after the ordering has been
	 *        applied
	 * @param countLimit
	 *        the count of items returned in the result list
	 * @return a {@link Map} containing the results with the properties defined in
	 *         {@link MonitorListProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public Map<String, Object> getMonitorList(Date startAfter, Date startBefore, String orderProperty,
											  String orderDirection, int countOffset, int countLimit)
		throws UciException;
	
	/**
	 * This request allows the UCI client to remove a monitor list entry from the list
	 * 
	 * @param monitorId
	 *        the id of the monitor list entry to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteMonitorListEntry(String monitorId) throws UciException;
	
	/**
	 * This request allows the UCI client to remove multiple monitor list entries from the list
	 * 
	 * @param monitorIds
	 *        the ids of the monitor list entries to remove
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void deleteMonitorListEntries(List<String> monitorIds) throws UciException;
	
	/**
	 * This request creates and returns a temporary URL that can be used to download the monitor
	 * file for the next 15 minutes.
	 * 
	 * @param monitorId
	 *        the id of the monitor list entry to retrieve the file for
	 * @return The download URL as {@link String}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getMonitorFileUrl(String monitorId) throws UciException;
	
	/**
	 * Prepares the download of the monitor for the given monitor id. After a calling this request
	 * the file can be downloaded using the interface {@link MonitorFileDownload} and the returned
	 * fileId.
	 * 
	 * @param monitorId
	 *        the id of the monitor list entry to retrieve the file for
	 * @return the fileId that can be used to download the file from STARFACE
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public String getMonitorFile(String monitorId) throws UciException;
	
	/**
	 * This interface defines the RPC methods that can be used to download the voicemail files
	 */
	@RpcInterface(SERVICE_NAME + ".getMonitorFile.rpcFileDownload")
	public interface MonitorFileDownload extends RpcFileDownload
	{
		// Just mapping the procedures from the super interface with the correct service name
	}
}
